#--
#
#################################################################################
# Copyright 2011 CENATIC y IEPALA
#
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas por la Comisión Europea– 
# versiones posteriores de la EUPL (la «Licencia»);
# Solo podrá usarse esta obra si se respeta la Licencia.
# Puede obtenerse una copia de la Licencia en:
#
# http://www.osor.eu/eupl/european-union-public-licence-eupl-v.1.1
#
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye «TAL CUAL»,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones que establece la Licencia.
#################################################################################
#
#++
# Controlador encargado de la gestión de los datos de proyecto.
# Este controlador es utilizado desde las secciones:
# * Sección proyectos: se utiliza para establecer la identificación, el contexto, los beneficiarios
# las relaciones, los objetivos y la sostenibilidad y las etapas del proyecto guardado en la session.




class DatosProyectoController < ApplicationController
  before_filter :verificar_usuario

  def verificar_usuario
    if ( session[:usuario_identificado].send( params[:seccion] ) == true and ( params[:seccion] == "proyectos" ) )
      case params[:action]
      when "anadir_moneda", "anadir_implementador", "anadir_financiador", "anadir_sector_poblacion", "anadir_sector_intervencion", "anadir_area_actuacion", "anadir_libro", "anadir_datos_proyecto_dinamicos", "modificar_datos_proyecto_dinamicos"
        then
        if ! ( params[:seccion] == "proyectos" and @proyecto.usuario_x_proyecto.find_by_usuario_id( session[:usuario_identificado] ).rol == "coordinador" ) 
          render :partial => "derecho_insuficiente"
        end          
      end
    else    
      flash[:mensaje] = _("No tiene los derechos suficiente para acceder")
     redirect_to "/inicio/tarea/inicio"
    end
  end

  before_filter :verificar_estado_proyecto, :only => [ :index, :modificar_beneficiarios, :modificar_crear_etapa, :eliminar_etapa, :modificar_datos_proyecto_dinamicos , :eliminar_datos_dinamicos]

  def verificar_estado_proyecto
    unless @proyecto.estado_actual && @proyecto.estado_actual.definicion_estado.formulacion
      flash[:error] = _("El proyecto se encuentra 'sin estado'. En este estado no se pueden modificar los datos del proyecto.")    if @proyecto.estado_actual.nil?
      flash[:error] = _("El proyecto se encuentra en estado de ") +  @proyecto.estado_actual.definicion_estado.nombre + _(". En este estado no se pueden modificar los datos del proyecto. No ha sido definido como 'estado de formulación' por su administrador.")    unless @proyecto.estado_actual.nil?
      redirect_to :action => "beneficiarios" if params[:action] == "modificar_beneficiarios"
      redirect_to :action => "etapas" if params[:action] =~ /modificar_crear_etapa|eliminar_etapa/
      redirect_to :action => "datos_proyecto_dinamicos_" + params[:pestana] if params[:action] =~ /modificar_datos_proyecto_dinamicos|eliminar_datos_dinamicos/
    end     
  end

  before_filter :verificar_estado_proyecto_ajax, :only => [ :relacionar, :eliminar_relacion,  :modificar_dato_dinamico]


  def verificar_estado_proyecto_ajax
    unless @proyecto.estado_actual && @proyecto.estado_actual.definicion_estado.formulacion
      # A no ser que queramos incluir una cuenta nueva, no dejamos modificar 
      unless !@proyecto.estado_actual.definicion_estado.cerrado && params[:atributo] == "libro" && params[:action] == "relacionar"
        render :update do |page|
          page.replace_html "MB_content", :inline => "<%= mensaje_error ('El proyecto se encuentra en estado de ' + @proyecto.estado_actual.definicion_estado.nombre + '. En este estado no se puede modificar los datos del proyecto. No ha sido definido como estado de formulación por su administrador.')  %>" if @proyecto.estado_actual
          page.replace_html "MB_content", :inline => "<%= mensaje_error ('El proyecto se encuentra sin estado. En este estado no se puede modificar los datos del proyecto.')  %>" unless @proyecto.estado_actual
          page.call("Modalbox.resizeToContent")
        end 
      end
    end
  end


  # en proyectos: se redirecciona por defecto a identificación
  def index
    redirect_to :action => "datos_proyecto_dinamicos_identificacion", :pestana => "identificacion"
  end

  # en proyectos:  lista los datos de beneficiarios
  def beneficiarios
    if @proyecto.datos_proyecto
      @datos_proyecto = @proyecto.datos_proyecto.reload
    else
      @datos_proyecto = @proyecto.create_datos_proyecto
    end
    @rotulo_atributo = [["Titulares de Derecho Directos Hombres", "beneficiarios_directos_hombres"],
                        ["Titulares de Derecho Directos Mujeres", "beneficiarios_directos_mujeres"],
                        ["Titulares de Derecho Indirectos Hombres", "beneficiarios_indirectos_hombres"],
                        ["Titulares de Derecho Indirectos Mujeres", "beneficiarios_indirectos_mujeres"],
                        ["Titulares de Derecho Directos Sin Especificar", "beneficiarios_directos_sin_especificar"],
                        ["Titulares de Derecho Indirectos Sin Especificar", "beneficiarios_indirectos_sin_especificar"],
                        ["Población Total de la Zona", "poblacion_total_de_la_zona"]]
    render :action => "beneficiarios"
  end

  # en proyectos: prepara el formulario de edición o creación de beneficiarios
  def editar_beneficiarios
    @datos_proyecto = DatosProyecto.find(params[:id])
    @atributos = [["Titulares de Derecho Directos Hombres", "beneficiarios_directos_hombres"],
                  ["Titulares de Derecho Directos Mujeres", "beneficiarios_directos_mujeres"],
                  ["Titulares de Derecho Indirectos Hombres", "beneficiarios_indirectos_hombres"],
                  ["Titulares de Derecho Indirectos Mujeres", "beneficiarios_indirectos_mujeres"],
                  ["Titulares de Derecho Directos Sin Especificar", "beneficiarios_directos_sin_especificar"],
                  ["Titulares de Derecho Indirectos Sin Especificar", "beneficiarios_indirectos_sin_especificar"],
                  ["Población Total de la Zona", "poblacion_total_de_la_zona"]]
    render :partial => "formulario_beneficiarios"
  end

  # en proyectos: modifica o crea beneficiarios
  def modificar_beneficiarios
    @datos_proyecto = DatosProyecto.find(params[:id])
    @datos_proyecto.update_attributes(params[:datos_proyecto] )
    @rotulo_atributo = [["Titulares de Derecho Directos Hombres", "beneficiarios_directos_hombres"],
                        ["Titulares de Derecho Directos Mujeres", "beneficiarios_directos_mujeres"],
                        ["Titulares de Derecho Indirectos Hombres", "beneficiarios_indirectos_hombres"],
                        ["Titulares de Derecho Indirectos Mujeres", "beneficiarios_indirectos_mujeres"],
                        ["Titulares de Derecho Directos Sin Especificar", "beneficiarios_directos_sin_especificar"],
                        ["Titulares de Derecho Indirectos Sin Especificar", "beneficiarios_indirectos_sin_especificar"],
                        ["Población Total de la Zona", "poblacion_total_de_la_zona"]]
    flash[:error] = @datos_proyecto
    redirect_to :action => "beneficiarios"
  end

  # en proyectos: lista las relaciones del proyectos
  def relaciones
    @paises = @proyecto.pais.all(:order => "nombre")
    @monedas = @proyecto.moneda.all(:order => "nombre")
    @financiadores = @proyecto.financiador.all(:order => "nombre")
    @implementadores = @proyecto.implementador.all(:order => "nombre")
    @sector_poblacion = @proyecto.sector_poblacion
    @sector_intervencion = @proyecto.sector_intervencion
    @area_actuacion = @proyecto.area_actuacion
    @libro = @proyecto.libro.all(:order => "nombre")
    @proyecto_cofinanciador = @proyecto.proyecto_cofinanciador.all(:order => "nombre")
  end

  # en proyectos: prepara el formulario para añadir un pais
  def anadir_pais
    @relaciones = (Pais.find(:all, :order => "nombre")-@proyecto.pais).collect{ |p| [p.nombre, p.id]}    
    render :partial => "anadir_relacion", :locals => { :relacion => "pais" }
  end

  # en proyectos: prepara el formulario para añadir una moneda
  def anadir_moneda
    @relaciones = (Moneda.find(:all, :order => "nombre")-@proyecto.moneda).collect{ |m| [m.abreviatura + " " + m.nombre, m.id]}    
    render :partial => "anadir_relacion", :locals => { :relacion => "moneda" }
  end

  # en proyectos: prepara el formulario para añadir un agente implementador
  def anadir_implementador 
    @relaciones = (Agente.find_all_by_implementador( true, :order => "nombre" )-@proyecto.implementador).collect{ |m| [m.nombre, m.id]} 
    render :partial => "anadir_relacion", :locals => { :relacion => "implementador" }
  end

  # en proyectos: prepara el formulario para añadir un agente financiador
  def anadir_financiador
    @relaciones = (Agente.find_all_by_financiador( true, :order => "nombre" )-@proyecto.financiador).collect{ |m| [m.nombre, m.id]}  
    render :partial => "anadir_relacion", :locals => { :relacion => "financiador" }
  end

  # en proyectos: prepara el formulario para añadir un sector de población
  def anadir_sector_poblacion
    @relaciones = SectorPoblacion.find( :all, :order => "nombre" ).collect{ |m| [m.nombre, m.id]} 
    render :partial => "anadir_relacion", :locals => { :relacion => "sector_poblacion" }
  end

  # en proyectos: prepara el formulario para añadir un sector de intervención
  def anadir_sector_intervencion
    @relaciones = SectorIntervencion.find( :all, :order => "nombre" ).collect{ |m| [m.nombre, m.id]} 
    render :partial => "anadir_relacion", :locals => { :relacion => "sector_intervencion" }
  end

  # en proyectos: prepara el formulario para añadir una area de actuación
  def anadir_area_actuacion
    @relaciones = AreaActuacion.find( :all, :order => "nombre" ).collect{ |m| [m.nombre, m.id]} 
    render :partial => "anadir_relacion", :locals => { :relacion => "area_actuacion" }
  end

  # en proyectos: prepara el formulario para añadir una cuenta (libro)
  def anadir_libro
    l=[]
    libros=[]
    # Obtiene todos los libros de todos los usuarios involucrados
    @proyecto.usuario.each { |usuario| l += usuario.libro }
    # Le quita los ya asignados, elimina duplicados y coge solo de los que la moneda este definida y el agente tambien
    (l-@proyecto.libro).each {|lib| libros << lib if libros.count(lib) == 0 && @proyecto.moneda.find_by_id(lib.moneda_id) && @proyecto.implementador.find_by_id(lib.agente_id) }
    # Ordena y prepara el desplegable 
    @relaciones = libros.sort! {|a,b| a.nombre <=> b.nombre}.collect{ |m| [m.nombre, m.id]}
    render :partial => "anadir_relacion", :locals => { :relacion => "libro" }
  end


  # Define un hash de relación atributo/modelo
  RELACIONES = { "pais" => "Pais", "moneda" => "Moneda", "implementador" => "Agente", "financiador" => "Agente", "sector_poblacion" => "SectorPoblacion", "sector_intervencion" => "SectorIntervencion", "area_actuacion" => "AreaActuacion", "libro" => "Libro" }

  # en proyecto: modifica o añade una relación 
  def relacionar
    begin 
      @proyecto.errors.clear
      @proyecto.send( params[:atributo] ) << eval(RELACIONES[params[:atributo]]).find(params[:relacion][:relacion_id])
    rescue
      @proyecto.errors.add(params[:atributo],"La relación no es correcta.")
    end
    @relaciones = @proyecto.send params[:atributo]
    @relacion = eval(RELACIONES[params[:atributo]]).find(params[:relacion][:relacion_id])

    render :update do |page|
      page.replace_html 'formulario', :inline => '<%= mensaje_error(@proyecto) %>'
      page.replace_html params[:atributo], :partial => "datos", :locals => { :relacion => params[:atributo] , :relaciones => @relaciones}
      page.visual_effect :highlight, params[:atributo] , :duration => 6
      # Si la relacion es pais, actualizamos tambien el listado de monedas por si hubo cambios
      if params[:atributo] == "pais"
        page.replace_html "moneda", :partial => "datos", :locals => { :relacion => "moneda", :relaciones => @proyecto.moneda}
        page.visual_effect :highlight, "moneda", :duration => 6
      end
      page.call("Modalbox.resizeToContent")
    end
  end

  # en proyectos: elimina una relación
  def eliminar_relacion
    # Usamos delete para no borrar el objeto, sino solo la relacción
    @proyecto.send( params[:relacion] ).delete( eval( RELACIONES[ params[:relacion] ] ).find( params[:id] ) )
    @relaciones = @proyecto.send params[:relacion]
    render :update do |page|
      #page.replace params[:relacion], :partial => "datos", :locals => { :relacion => params[:relacion] , :relaciones => @relaciones}
      page.replace_html params[:relacion], :partial => "datos", :locals => { :relacion => params[:relacion] , :relaciones => @relaciones}
      page.visual_effect :highlight, params[:relacion] , :duration => 6
      page.replace_html 'MB_content', :inline => '<%= mensaje_error(@proyecto, :eliminar => true) %><br>'
      page.call("Modalbox.resizeToContent")
    end
  end
  
  # --
  ######### DATOS PROYECTO DINAMICOS ###########
  # ++

  # :en proyecto, lista los datos dinamicos del proyecto según la pestaña
  def datos_proyecto_dinamicos_borrame
    params[:pestana] ||= "identificacion"
    case params[:pestana]
      when "identificacion" then @datos_dinamicos = @proyecto.datos_dinamicos "identificacion"
      when "contexto"      then @datos_dinamicos = @proyecto.datos_dinamicos "contexto"
      when "objetivos"      then @datos_dinamicos = @proyecto.datos_dinamicos "objetivos"
    end
  end

  def datos_proyecto_dinamicos_identificacion
    @datos_dinamicos = @proyecto.datos_dinamicos "identificacion"
    @pestana = "identificacion"
    render "datos_proyecto_dinamicos"
  end

  def datos_proyecto_dinamicos_contexto
    @datos_dinamicos = @proyecto.datos_dinamicos "contexto"
    @pestana = "contexto"
    render "datos_proyecto_dinamicos"
  end

  def datos_proyecto_dinamicos_objetivos
    @datos_dinamicos = @proyecto.datos_dinamicos "objetivos"
    @pestana = "objetivos"
    render "datos_proyecto_dinamicos"
  end

  # prepara el formulario de edición de un dato dinamico
  def editar_dato_dinamico
    @dato_dinamico = eval("Dato" + params[:tipo]).find( params[:dato_dinamico_id] )
    render :partial => "dato_dinamico", :pestana => params[:pestana]
  end

  def modificar_dato_dinamico
    @dato_dinamico = eval( "Dato" + params[:tipo]).find( params[:dato_dinamico_id] )
    @dato_dinamico.update_attribute( :dato, params[:dato_dinamico][:dato] )
    flash[:error] = @dato_dinamico
    @pestana = params[:pestana] # chapu para pasarle la pestaña
    render :update do |page|
      page.replace_html 'formulario', :inline => '<%= mensaje_error(@dato_dinamico) %><br>'
      page.call("Modalbox.resizeToContent")
      page.replace_html params[:update], :partial => "modificacion_dato_dinamico"
      page.visual_effect :highlight, params[:update] , :duration => 6

    end
  end
 
  # --
  ######### GESTIONAR DATOS PROYECTO DINAMICOS ###########
  # ++

  # :en proyecto, prepara la modal para añadir datos dinamicos según la pestaña
  def anadir_datos_proyecto_dinamicos
    @definiciones_dato = DefinicionDato.find_all_by_pestana( params[:pestana] ) - @proyecto.dato_texto.collect{|d| DefinicionDato.find_by_id d.definicion_dato_id}
    render :partial => "anadir_datos_proyecto_dinamicos", :pestana => params[:pestana]
  end

  def modificar_datos_proyecto_dinamicos
    params[:definicion_dato_ids ] ||= []
    params[:definicion_dato_ids].each do |def_dat|
      definicion_dato = DefinicionDato.find(def_dat)
      ProyectoXDefinicionDato.create :proyecto_id => @proyecto.id, :definicion_dato_id => def_dat
      eval( "Dato" + definicion_dato.tipo).create(:proyecto_id => @proyecto.id, :definicion_dato_id => def_dat)
    end

    
    redirect_to :action => "datos_proyecto_dinamicos_" + params[:pestana], :pestana => params[:pestana]
  end

  def eliminar_datos_dinamicos
    eval( "Dato" + DefinicionDato.find(params[:definicion_dato_id]).tipo).find(params[:dato_id]).destroy
    ProyectoXDefinicionDato.find_by_proyecto_id_and_definicion_dato_id(@proyecto.id, params[:definicion_dato_id]).destroy
    redirect_to :action => "datos_proyecto_dinamicos_" + params[:pestana], :pestana => params[:pestana]
  end

  # --
  ########## ETAPAS ##########
  # ++

  # en proyectos: lista las etapas
  def etapas
    @etapa = params[:id] ?  Etapa.find(params[:id]) : nil
    @etapas = @proyecto.etapa.reload.sort! { |a,b| a.fecha_inicio <=> b.fecha_inicio }
    render "comunes/etapas"
  end

  # en proyectos: crea una nueva etapa
  def editar_nuevo_etapa
    @etapa = params[:id] ?  Etapa.find(params[:id]) : nil
    render :partial => "comunes/etapa"
  end

  # en proyectos: modifica o crea una etapa
  def modificar_crear_etapa
    @etapa = params[:id] ?  Etapa.find(params[:id]) : Etapa.new(:proyecto_id => @proyecto.id)
    fecha_inicio_anterior = @etapa.fecha_inicio
    @etapa.update_attributes params[:etapa]
    actualizar_cronograma_presupuesto (fecha_inicio_anterior != @etapa.fecha_inicio)  if params[:id] #Si ya existia la etapa y se ha modificado
    actualizar_cronograma_actividad  if params[:id] #Si ya existia la etapa y se ha modificado
    flash[:error] = @etapa
    redirect_to :action => 'etapas'
  end
    
  # Al cambiar las fechas de una etapa actualizamos todos los presupuestos_detallados de los presupuestos asociados a dicha etapa
  def actualizar_cronograma_presupuesto cambia_inicio
    @etapa.presupuesto.each do |p|
      p.presupuesto_detallado.each do |pd|
        (pd.destroy; next) if pd.mes > @etapa.meses #Borramos el detalle cuyo mes sea mayor que el nuevo numero de meses
        if cambia_inicio #Si ha cambiado la fecha de inicio
          pd.update_attribute("fecha_inicio", (@etapa.fecha_inicio >> (pd.mes - 1)))
          pd.update_attribute("fecha_fin", ((pd.fecha_inicio >> 1) - 1)) 
        end
        pd.update_attribute("fecha_fin", @etapa.fecha_fin) if pd.fecha_fin > @etapa.fecha_fin #Actualizamos a la ultima fecha de fin
      end
    end
  end

  # Al cambiar las fechas de una etapa actualizamos todas las actividad_detalladas de las actividades asociadas a dicha etapa
  def actualizar_cronograma_actividad
    @etapa.actividad.each do |a| 
      a.actividad_detallada.each do |ad|
        (ad.destroy; next) if ad.mes > @etapa.meses #Borramos el detalle cuyo mes sea mayor que el nuevo numero de meses
      end
    end
  end


  # en proyectos: elimina una etapa
  def eliminar_etapa
    (etapa = Etapa.find(params[:id])).destroy
    flash[:error] = etapa
    redirect_to :action => 'etapas'
  end

  # --
  ########## PROYECTOS CONFINANCIADORES ##########
  # ++

  # en proyectos: lista los proyectos cofinanciadores
  def proyecto_cofinanciador
    @proyectos_cofinanciadores = @proyecto.proyecto_x_proyecto
  end

  # en proyectos: crea un nuevo proyecto cofinanciador
  def editar_nuevo_proyecto_confinanciador
    @proyectos = Proyecto.all.collect{|p| [p.nombre, p.id] unless p.convenio?}.compact
    @proyecto_x_proyecto = ProyectoXProyecto.find_by_id(params[:id]) || ProyectoXProyecto.new
    render :partial => "formulario_proyecto_cofinanciador"
  end

  # en proyectos: modifica o crea un proyecto cofinanciador
  def modificar_crear_proyecto_cofinanciador
    proyecto_cofinanciador = @proyecto.proyecto_x_proyecto.find_by_id(params[:id]) || ProyectoXProyecto.new(:proyecto_id => @proyecto.id)
    proyecto_cofinanciador.update_attributes params[:proyecto_x_proyecto]
    flash[:objeto_modificado] = {:errors => proyecto_cofinanciador.errors }
    redirect_to :action => 'proyecto_cofinanciador'
  end

  # en proyectos: elimina un proyecto cofinanciador
  def eliminar_proyecto_cofinanciador
    proyecto_cofinanciador = @proyecto.proyecto_x_proyecto.find_by_id params[:id] 
    proyecto_cofinanciador.destroy
    flash[:error] = proyecto_cofinanciador
    redirect_to :action => 'proyecto_cofinanciador'
  end

end
#done

