#--
#
#################################################################################
# Copyright 2011 CENATIC y IEPALA
#
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas por la Comisión Europea– 
# versiones posteriores de la EUPL (la «Licencia»);
# Solo podrá usarse esta obra si se respeta la Licencia.
# Puede obtenerse una copia de la Licencia en:
#
# http://www.osor.eu/eupl/european-union-public-licence-eupl-v.1.1
#
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye «TAL CUAL»,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones que establece la Licencia.
#################################################################################
#
#++
# Controlador encargado de la gestión de los documentos.
# Este controlador es utilizado desde las secciones:
# * Sección proyectos: se utiliza para listar, comentar, subir y descargar documentos
# * Sección financiaciones: se utiliza para listar, comentar, subir y descargar documentos
class DocumentoController < ApplicationController

  # Solo elimina cuando el documento pertenece
  before_filter :autorizar, :only => [ :eliminar, :modificar_crear ]
  before_filter :busca_etiquetas, :only => [ :anadir_filtro, :listado ]

	# en proyectos y en financiación: se redirecciona por defecto a ordenado
  def index
    #session[:documento_filtro] = etiqueta_todos 
    redirect_to :action => "ordenado"
  end

  # Hace un filtrado segun las etiquetas que tengamos seleccionadas
  def filtrado
    # Primero vaciamos la variable del filtros (hay una por seccion)
    session[("filtro_etiquetas_" + params[:seccion]).to_sym] = []
    # Incluimos el valor del selector si hay algo seleccionado 
    session[("filtro_etiquetas_" + params[:seccion]).to_sym] << params[:selector][:etiqueta_nueva] if params[:selector][:etiqueta_nueva].to_i != 0
    # Vamos incluyendo las etiquetas seleccionadas previamente
    if params[:filtro_etiqueta]
      params[:filtro_etiqueta].each_value do |id|
        session[("filtro_etiquetas_" + params[:seccion]).to_sym] << id if id != "" && id.to_i > 0
      end 
    end
    redirect_to :action => :listado
  end

  # Incluye la etiqueta seleccionada para poder filtrar
  def anadir_filtro
    render :update do |page|
      page.replace_html params[:update], :partial => "elemento_filtrado", :locals => { :indice => params[:indice].to_i, :valor => params[:selector][:etiqueta_nueva], :ultimo => true }
    end
  end

  # Elimina la etiqueta seleccionada de las elegidas para filtrar
  def quitar_filtro
    render :update do |page|
      page.replace_html params[:update], :inline => ''
    end
  end

	# en proyectos y en financiación: establece los parametros de ordenación
  def ordenado
    session[:documento_asc_desc] = params[:asc_desc] ? params[:asc_desc] : "ASC" 
    session[:documento_orden] = params[:orden] ? params[:orden] : "adjunto_file_name" 
    redirect_to :action => "listado" if params[:seccion] == "proyectos"
    redirect_to :action => "iniciar_listado_por_espacio" if params[:seccion] == "documentos"
    redirect_to :action => "listado" if params[:seccion] == "administracion"
  end

	# en proyectos y en financiación: lista los documentos disponibles
  def listado
    documentos_asociados
  end

  
	# en proyectos y en financiación: prepara el formulario de edición o creación
  def editar_nuevo
    if params[:documento]
      # Todo esto hay que hacerlo por el iframe de subida con ajax
      params[:tipo] ||= params[:url][:tipo] if params[:url]
      params[:objeto_id] ||= params[:url][:objeto_id] if params[:url]
      params[:i] ||= params[:url][:i] if params[:url]
      modificar_crear
    else
      @documento = params[:id] ?  Documento.find(params[:id]) : Documento.new
      carga_espacios_etiquetas
      espacio = Espacio.find session[:espacio_seleccionado] if session[:espacio_seleccionado] and params[:seccion] == "documentos" and params[:listado] == "listado_por_espacio"
      render :update do |page|
        # Si se estamos navegando por espacios no dejamos que se pueda tocar espacios de proyeto o espacios de plantillas
        if espacio and (espacio.definicion_espacio_proyecto_id or espacio.proyecto_id or espacio.nombre == "Plantillas exportación")
          page.mensaje_informacion params[:update], "No se puede incluir documentación para espacios de proyectos o de administración desde esta sección.", :tipo_mensaje => "mensajefallo" 
        else
           page.formulario :partial => "formulario", :update => params[:update]  unless params[:pisa_formulario]
           page.replace 'formulariocontenedor', :partial => "formulario", :locals => { :update => params[:update] }  if params[:pisa_formulario]
        end
      end
    end
  end

	# en proyectos y en financiación:  modifica o crea
  def modificar_crear
    @documento = Documento.find_by_id(params[:id]) || Documento.new
    # Solo ponemos el usuario_id si no existe o el documento es nuevo
    params[:documento][:usuario_id] = session[:usuario_identificado].id if !params[:id] || !@documento.usuario_id
    @documento.update_attributes params[:documento]
    #@proyecto.espacio.documento << @documento if params[:seccion] == "proyectos"
    # Si estamos en administracion incluimos en documento en el espacio de plantillas de exportación
    if params[:seccion] == "administracion"
      espacio = Espacio.find_by_nombre "Plantillas exportación"
      espacio.documento << @documento
    end
    # Para cambiar un documento de espacio dentro de proyectos
    if params[:seccion] == "proyectos" and params[:cambiar] and params[:cambiar][:espacio] == "1" and params[:cambiar][:espacio_id] != ""
      @documento.update_attribute "espacio_id", params[:cambiar][:espacio_id] 
    end
    @documento.etiqueta.clear
    @documento.etiqueta_ids = params["etiqueta"].to_a.collect {|p| p[0] if p[1] == "1"} if @documento.errors.empty?
    # Si estamos trabajando en las fuentes de verificacion vinculamos el documento a la fuente de verificacion
    #tipo, objeto_id = obtiene_tipo_y_objeto
    if params[:tipo] && params[:tipo] != ""
      logger.info ">>>>>>>>>>>>>>> Asociamos Documento a un objeto " + params[:tipo] + " <<<<<<<<<<<<<<<<<<"
      objeto = eval( params[:tipo] ).find_by_id(params[:objeto_id].to_i)
      objeto.documento << @documento unless objeto.nil? || objeto.documento.find_by_id(@documento.id)
    end
    if @documento.errors.empty?
      responds_to_parent do
        # Si estamos en el listado de documentos
        unless params[:tipo] && params[:tipo] != ""
          i = params[:i] || params[:url][:i]
          render :update do |page|
            if params[:id]
              page.modificar(:update => params[:update], :partial => "documento", :mensaje => { :errors => @documento.errors })
            else
              page.show "nuevos_documentos"
              page.modificar(:update => "documento_nuevo_" + params[:i], :partial => "nuevo_documento", :mensaje => { :errors => @documento.errors })
              page.replace "anadir", :inline => "<%= anadir(:url => {:action => 'editar_nuevo', :i => (params[:i].to_i + 1).to_s, :update => 'formulario_anadir'}) %>"
            end
          end
        # Si es una vinculacion a un documento recargamos todo el listado
        else
          documentos_asociados
          # Para FV
          #render(:update){ |page|  page.modificar(:update_listado => params[:update_listado], :locals => {:update => params[:update_listado], :objeto_id => params[:objeto_id], :documentos => @documentos}, :partial => "fuente_verificacion/listado_documentos", :mensaje => { :errors => @documento.errors}) } if params[:tipo] == "FuenteVerificacion"
          # Para otros objetos (en sublistado)
          render(:update){ |page|  page.modificar(:update_listado => params[:update_listado], :partial => "listado_asociados", :locals => {:objeto_id => params[:objeto_id], :tipo => params[:tipo]}, :mensaje => { :errors => @documento.errors}, :tipo_update => "sublistado") } 
        end
      end
    else
      # Si hay datos que fallan se vuelve al formulario
      carga_espacios_etiquetas
      responds_to_parent do
        render(:update){ |page|  page.recargar_formulario :partial => "formulario", :mensaje => { :errors => @documento.errors } }
      end
    end
  end    

	# en proyectos y en financiación: elimina documento
  def eliminar
    @documento = Documento.find(params[:id])
    @documento.destroy
    @proyecto if params[:seccion] == "proyectos"
    render(:update) { |page|  page.eliminar(:update => params[:update_listado], :mensaje => { :errors => @documento.errors, :eliminar => true }) }
  end

  def descargar
    @documento = Documento.find(params[:id])
      # file = File.new(@document.document.path, "r")
      # send_data(file.read,
      #  :filename     =>  @document.document_file_name,
      #  :type         =>  @document.document_content_type,
      #  :disposition  =>  'inline') 

    # Antes de descargar, comprobar que exista el fichero y marcar el error si no esta
    if File.exists?(@documento.adjunto.path)       
      send_file @documento.adjunto.path, :type => @documento.adjunto_content_type, :disposition => 'inline'
    else
      flash[:error] = _("No se pudo encontrar el documento. Contacte con el administrador del sistema.")
      redirect_to :action => 'listado'
    end
  end

  def seleccionar_espacio
    session[:espacio_seleccionado] = params[:id]  if params[:seccion] == "documentos"
    session[:espacio_proyecto_seleccionado] = params[:id] if params[:seccion] == "proyectos"
    redirect_to :action => :listado_por_espacio
  end


  def iniciar_listado_por_espacio
    session[:espacio_seleccionado] = nil  if params[:seccion] == "documentos"
    session[:espacio_proyecto_seleccionado] = @proyecto.espacio.id if params[:seccion] == "proyectos"
    redirect_to :action => :listado_por_espacio
  end

  def listado_por_espacio
    @espacio = Espacio.find(session[:espacio_seleccionado]) if session[:espacio_seleccionado] and params[:seccion] == "documentos"
    @espacio = Espacio.find(session[:espacio_proyecto_seleccionado]) if params[:seccion] == "proyectos"
    @documentos = Documento.paginate(:all, :conditions => {:espacio_id => @espacio}, :page => params[:page], :per_page => (session[:por_pagina] or 20))
    condiciones = ['espacio_padre_id = ? and (definicion_espacio_proyecto is NULL or definicion_espacio_proyecto = "0") ', @espacio.id ] if @espacio
    condiciones = 'espacio_padre_id is NULL and (definicion_espacio_proyecto is NULL or definicion_espacio_proyecto = "0") ' unless @espacio
    @espacios = Espacio.find :all, :conditions => condiciones
  end


        # muestra los documentos vinculados a un objeto
  def listado_asociados
    documentos_asociados
    render :update do |page|
      page.replace_html params[:update], :partial => "listado_asociados", :locals => { :objeto_id => params[:objeto_id], :tipo => params[:tipo], :documentos => @documentos, :update_listado => params[:update] }
    end
  end

  def asociar_o_nuevo
    render(:update) { |page| page.formulario :partial => "formulario_asociar_o_nuevo", :tipo => params[:tipo], :objeto_id => params[:objeto_id], :update => params[:update] }
  end

  def asociar_editar
    @documentos = @proyecto.documento.collect {|d| [d.adjunto_file_name, d.id]}
    @documentos = Espacio.find_by_espacio_padre_id_and_fuente_verificacion(@proyecto.espacio.id, true).documento.collect {|d| [d.adjunto_file_name, d.id]} if params[:tipo] == "FuenteVerificacion"
    @documentos = Espacio.find_by_espacio_padre_id_and_gasto(@proyecto.espacio.id, true).documento.collect {|d| [d.adjunto_file_name, d.id]} if params[:tipo] == "Gasto"

    render(:update) { |page| page.replace 'formulariocontenedor', :partial => "formulario_asociar", :locals => { :update => params[:update] } }
  end

  def asociar_objeto
    @objeto = eval( params[:tipo] ).find_by_id(params[:objeto_id])
    @relacion = eval(params[:tipo] + "XDocumento").create( params[:tipo].tableize + "_id" => params[:objeto_id], "documento_id" => params[:documento][:id] )

    if @relacion.errors.empty?
      documentos_asociados
      render(:update){ |page|  page.modificar(:update_listado => params[:update_listado], :partial => "listado_asociados", :locals => {:objeto_id => params[:objeto_id], :tipo => params[:tipo]}, :mensaje => { :errors => @relacion.errors}, :tipo_update => "sublistado") }
    else
      # Si hay datos que fallan se vuelve al formulario
      @documentos = @proyecto.documento.collect {|d| [d.adjunto_file_name, d.id]}
      render(:update){ |page|  page.recargar_formulario :partial => "formulario_asociar", :mensaje => { :errors => @relacion.errors } }
    end
  end
  
  def desasociar_objeto
    @objeto = eval( params[:tipo] ).find_by_id(params[:objeto_id])
    @objeto.documento.delete(Documento.find_by_id(params[:id]))
    documentos_asociados 
    render(:update) { |page|  page.eliminar(:update => params[:update_listado], :mensaje => { :errors => @objeto.errors, :eliminar => true }) }
  end


  # --
  ########## filtros ##########
  # ++
  private
    # Permitimos modificar y borrar solo al propietario del documento y al coordinador
    def autorizar
      if params[:id]
        @documento = Documento.find(params[:id])
        if @documento.usuario_id && @documento.usuario_id != session[:usuario_identificado].id && ((@proyecto && @proyecto.usuario_x_proyecto.find_by_usuario_id( session[:usuario_identificado].id ).rol != "coordinador") || session[:usuario_identificado].administracion )
          flash[:error] = ( _("Imposible: El documento no le pertenece.") )
          redirect_to :action => 'listado'
        end
      end
    end

    # Carga espacios y etiquetas
    def carga_espacios_etiquetas
      condiciones_espacio = "proyecto_id is NULL and nombre != 'Proyectos' and nombre != 'Plantillas exportación' and definicion_espacio_proyecto_id is NULL and definicion_espacio_proyecto is NULL "
      @espacios = Espacio.find(:all, :conditions => condiciones_espacio).collect {|e| [e.nombre , e.id]}
      # Si estamos en administracion y no existen las etiquetas de exportacion las creamos
      [ _("Exportacion Presupuesto"), _("Exportacion Gasto"), ("Exportacion Transferencia") ].each { |etiqueta| Etiqueta.new(:nombre => etiqueta, :descripcion => etiqueta, :tipo => "proyecto").save if Etiqueta.find_by_nombre(etiqueta).nil? } if params[:seccion] == "administracion"
      @etiquetas_exportacion = Etiqueta.find(:all, :conditions => ["etiqueta.nombre LIKE ?", _("Exportacion") + " %"])
      @etiquetas_proyecto = Etiqueta.find(:all, :conditions => {"tipo" => "proyecto"})
      @etiquetas_comunes = Etiqueta.find(:all, :conditions => {"tipo" => "comunes"})
      @etiquetas_documento = @documento.etiqueta.collect{ |e| e.id }
    end

    # Devuelve el conjunto de etiquetas valido para la seccion en la que estamos
    def busca_etiquetas
      condiciones_etiqueta = ""
      condiciones_etiqueta = "etiqueta.nombre LIKE ?", _("Exportacion") + " %" if params[:seccion] == "administracion"
      condiciones_etiqueta = "etiqueta.tipo = 'proyecto'" if params[:seccion] == "proyectos"
      @etiquetas = Etiqueta.find( :all, :conditions => condiciones_etiqueta ).collect{ |e| [e.nombre, e.id] }.sort{ |a, b| a[0] <=> b[0]}
    end

    # Devuelve el texto de seleccion de todas las etiquetas (o plantillas)
    def etiqueta_todos
      params[:seccion] == "administracion" ? _("Todas las plantillas") : _("Todas las etiquetas")
    end

    def obtiene_tipo_y_objeto
      tipo = params[:tipo] || (params[:url] ? params[:url][:tipo] : nil)
      objeto_id = params[:objeto_id] || (params[:url] ? params[:url][:objeto_id] : nil)
      tipo = nil if tipo == ""
      objeto_id = nil if objeto_id == ""
      return tipo,objeto_id
    end

    def documentos_asociados
      #tipo,objeto_id = obtiene_tipo_y_objeto
      # Obtiene los documentos asociados a un objeto
      if params[:tipo] && params[:tipo] != ""
        @documentos = []
        @objeto = eval( params[:tipo] ).find_by_id(params[:objeto_id])
        if params[:tipo] == "Movimiento"
          @documentos = @objeto.gasto.collect{|g| g.documento}.flatten if @objeto
        else
          @documentos = @objeto.documento if @objeto
        end
      # Obtiene los documentos cuando no se buscan asociados a objeto
      else
        # Establece las condiciones para la busqueda de documentos segun sea de proyecto o global
        condiciones_proyecto = ( @proyecto ? "espacio.proyecto_id = " + @proyecto.id.to_s : "" )
        # Si estamos en proyecto buscamos no solo en el espacio principal si no en los subespacios del proyecto
        for espacio in @proyecto.espacio.espacio_hijo
          condiciones_proyecto += " OR espacio.id = " + espacio.id.to_s
        end if @proyecto
  

        # Si tiene pacs (es un convenio) incluye en la busqueda los documentos de estos
        @proyecto.pacs.each do |pac|
          condiciones_proyecto += " OR espacio.proyecto_id = " + pac.id.to_s
        end if @proyecto
        condiciones_proyecto = "( " + condiciones_proyecto + " )" if condiciones_proyecto
        # Hace la busqueda de documentos segun tengamos filtros o no
        if session[("filtro_etiquetas_" + params[:seccion]).to_sym] and session[("filtro_etiquetas_" + params[:seccion]).to_sym].length > 0
          condicion_filtros = (params[:seccion] == "proyectos" ? condiciones_proyecto + " AND " : "") + "etiqueta_x_documento.etiqueta_id IN (?)", session[("filtro_etiquetas_" + params[:seccion]).to_sym]
          @documentos = Documento.paginate(:all, :include => ["etiqueta_x_documento", "espacio" ], :conditions => condicion_filtros, :group => ["documento.id"], :having => ["count(documento.id) >= " + session[("filtro_etiquetas_" + params[:seccion]).to_sym].length.to_s ], :order => (session[:documento_orden] + " " + session[:documento_asc_desc] ), :page => params[:page], :per_page => (session[:por_pagina] or 20) )
        else
          condicion_filtros = condiciones_proyecto if params[:seccion] == "proyectos"
          condicion_filtros = "etiqueta.nombre LIKE ?", _("Exportacion") + " %" if params[:seccion] == "administracion"
          condicion_filtros = nil if condicion_filtros == "(  )" # Chapucilla por un error cuando las condiciones estan vacias

          @documentos = Documento.paginate(:all, :include => [ "etiqueta_x_documento", "espacio", "etiqueta" ], :conditions => condicion_filtros, :order => (session[:documento_orden] + " " + session[:documento_asc_desc] ), :page => params[:page], :per_page => (session[:por_pagina] or 20) )
        end
      end
    end

end
