#--
#
#################################################################################
# Copyright 2011 CENATIC y IEPALA
#
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas por la Comisión Europea– 
# versiones posteriores de la EUPL (la «Licencia»);
# Solo podrá usarse esta obra si se respeta la Licencia.
# Puede obtenerse una copia de la Licencia en:
#
# http://www.osor.eu/eupl/european-union-public-licence-eupl-v.1.1
#
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye «TAL CUAL»,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones que establece la Licencia.
#################################################################################
#
#++
# Controlador encargado de la gestion de proyecto. Este controlador es utilizado desde las secciones:
# * Sección administracion: se utiliza para crear proyectos
# * Sección proyectos: se utiliza para seleccionar el proyecto y cargar lo en la sessión

class ProyectoController < ApplicationController
  # en administracion y proyectos: se redirecciona por defecto a ordenado
  def index
    redirect_to :action => 'ordenado'
  end

  # en administracion: lista
  def listado
    condiciones = "convenio_id IS NULL AND convenio_accion IS NULL" unless params[:menu] == "convenio"
    condiciones = "convenio_id IS NULL AND convenio_accion IS NOT NULL" if params[:menu] == "convenio"
    @proyectos = Proyecto.paginate( :all, :conditions => condiciones, :order => (session[:proyecto_orden] + " " + session[:proyecto_asc_desc] ) , :include => [:pais, :agente], :page => params[:page], :per_page => (session[:por_pagina] or 20) )
  end

  # en administracion y proyectos: establece los parametros de ordenación
  def ordenado
    session[:proyecto_asc_desc] = params[:asc_desc] ? params[:asc_desc] : "ASC"   
    session[:proyecto_orden] = params[:orden] ? params[:orden] : "nombre"

    redirect_to :action => "listado" if params[:seccion] == "administracion"
    redirect_to :action => "listado_usuario" if params[:seccion] == "proyectos"
  end
  
  # en administracion: prepara el formulario de edición o creación
  def editar_nuevo
    @proyecto = params[:id] ? Proyecto.find(params[:id]) : nil 
    @paises = Pais.all(:order => "nombre").collect {|p| [p.nombre, p.id]}
    @financiadores = Agente.find_all_by_financiador(true, :order => "nombre").collect {|a| [a.nombre, a.id]}
    @monedas = Moneda.all(:order => "nombre").collect {|a| [a.nombre, a.id]}
    @convenio_id = params[:convenio_id] if params[:convenio_id]
    render :partial => "formulario"
  end

  # en administracion: modifica o crea
  def modificar_crear
    @proyecto = params[:id] ?  Proyecto.find(params[:id]) : Proyecto.new
    @proyecto.update_attributes params[:proyecto]
    if params[:convenio_id]
      convenio = Proyecto.find_by_id params[:convenio_id]
      @proyectos = convenio.pacs
      render :update do |page|
        page.replace_html params[:update], :partial => "listado_proyectos_convenio"
        page.visual_effect :highlight, params[:update] , :duration => 6
        page.replace 'formulario', :inline => '<%= mensaje_error(@proyecto) %><br>'
        page.call("Modalbox.resizeToContent")
      end
    else
      flash[:error] = @proyecto
      redirect_to :action => "listado"
    end
  end

  # en administracion: elimina
  def eliminar
    @proyecto = Proyecto.find(params[:id])
    @proyecto.destroy
    if params[:convenio_id] 
      convenio = Proyecto.find_by_id params[:convenio_id]
      @proyectos = convenio.pacs
      render :update do |page|
        page.replace_html params[:update], :partial => "listado_proyectos_convenio"
        page.visual_effect :highlight, params[:update] , :duration => 6
        page.replace_html 'MB_content', :inline => '<%= mensaje_error(@proyecto, :eliminar => true) %><br>'
        page.call("Modalbox.resizeToContent")
      end
    else
      flash[:error_eliminar] = @proyecto
      redirect_to :action => 'listado'
    end
  end
  
  # en proyectos: lista los proyectos asociados al usuario declarado en la sessión
  def listado_usuario
    @proyectos = session[:usuario_identificado].proyecto.reload.paginate(:all, :conditions => "convenio_id IS NULL", :order => (session[:proyecto_orden] + " " + session[:proyecto_asc_desc] ) , :include => [:pais, :agente], :page => params[:page], :per_page => (session[:por_pagina] or 20) )
  end

  # en convenios: lista los proyectos asociados
  def listado_proyectos_convenio
    convenio = Proyecto.find_by_id params[:convenio_id]
    @proyectos = convenio.pacs
    render :update do |page|
      page.replace_html params[:update], :partial => "listado_proyectos_convenio"
    end

  end

end
