#--
#
#################################################################################
# Copyright 2011 CENATIC y IEPALA
#
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas por la Comisión Europea– 
# versiones posteriores de la EUPL (la «Licencia»);
# Solo podrá usarse esta obra si se respeta la Licencia.
# Puede obtenerse una copia de la Licencia en:
#
# http://www.osor.eu/eupl/european-union-public-licence-eupl-v.1.1
#
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye «TAL CUAL»,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones que establece la Licencia.
#################################################################################
#
#++
# Controlador encargado de las vistas resumen financieras para una agente implementador. Este controlador es utilizado desde las secciones agentes.

class ResumenAgenteController < ApplicationController

  helper :resumen_proyecto
  
  # en agente: se redirecciona por defecto a presupuesto
  def index
      if @agente.etapa.empty? 
        flash[:error] = _("Tiene que definir por lo menos una etapa")
        redirect_to :controller => :datos_agente, :action => :etapas
      else
        redirect_to :action => :presupuesto
      end
  end

  def gasto
     @listado_moneda = @agente.moneda.collect{|e| [e.nombre, e.id]} + [["Todas las monedas (con tasa aplicada)","todas"]]
    # Hacemos la busqueda si se seleccionan los criterios
    estado_gasto if params[:moneda]
  end

  def estado_gasto
    @fecha_de_inicio = fecha params["fecha_inicio"]
    @fecha_de_fin = (fecha(params["fecha_fin"]) >> 1) -1 # Lo adelantamos un mes para que sea mas coherente

    # Recogemos la tabla de resumen general
    condiciones = {"ocultar_agente" => false} unless params[:mostrar_partidas_ocultas] == "1"
    filas = Partida.find(:all, :conditions => condiciones, :order => "codigo").collect{|i| {"id" => i.id, "nombre" => i.codigo_nombre}}
    columnas = [{"id" => "1", "nombre" => "Presupuesto"}, {"id" => "2", "nombre" => "Gastos"}]
#    presupuestos = VPresupuestoDetallado.partida_de_implementador_sin_proyecto(params[:moneda], @agente.id, @fecha_de_inicio, @fecha_de_fin , "0").collect {|p| {"columna_id" => "1", "importe" => p.importe , "fila_id" => p.fila_id }}
    presupuestos = VPresupuestoAgenteDetallado.partidas(@agente.id, @fecha_de_inicio, @fecha_de_fin , params[:moneda],  params[:tasa_cambio]).collect {|p| {"columna_id" => "1", "importe" => p.importe , "fila_id" => p.partida_id.to_s }}
    gastos = VGastoAgente.partida_implementador(@agente.id, @fecha_de_inicio, @fecha_de_fin, params[:moneda], params[:tasa_cambio], nil).collect {|p| {"columna_id" => "2", "importe" => p.importe , "fila_id" => p.partida_id.to_s }}
    datos = presupuestos + gastos
    titulo_datos =[ "Moneda: #{ params[:moneda] == 'todas' ?  'Todas las monedas' : Moneda.find(params[:moneda]).abreviatura} " ,
               "Tasa cambio: #{(params[:tasa_cambio] == '1' or params[:moneda] == 'todas') ? 'Aplicada': 'No aplicada'}" ,
               "Fechas: #{  @fecha_de_inicio.to_s + '/' + @fecha_de_fin.to_s } " ]

    titulo_resumen =  ["Resumen de gasto comparado con el presupuesto."] + titulo_datos
    tabla_resumen = { :titulo => titulo_resumen, :filas => filas, :columnas => columnas, :datos => datos, :otros => { :fila_suma => true, :columna_resta => true, :columna_desviacion => true } }

     # Comprobamos que no hay partidas ocultas con datos de agente para comunicarlo al usuario
     presupuestos.each do |p|
       flash.now[:error] = "Existen partidas ocultas que contienen presupuestos. Seleccione mostrar partidas ocultas para verlas"  if (filas.select{|f| f["id"].to_s == p["fila_id"].to_s}).empty?
     end
     gastos.each do |p|
       flash.now[:error] = "Existen partidas ocultas que contienen gastos. Seleccione mostrar partidas ocultas para verlas"  if (filas.select{|f| f["id"].to_s == p["fila_id"].to_s}).empty?
     end

     # Recogemos el listado de subpartidas de presupuesto y gastos.
     @v_presupuesto_subpartida = VPresupuestoAgenteDetallado.subpartidas(@agente.id,  @fecha_de_inicio, @fecha_de_fin, params[:moneda], params[:tasa_cambio])
     @titulo_presupuesto_subpartida = ["Detalle del presupuesto del agente por subpartidas"] + titulo_datos

     @v_gasto_subpartida = VGastoAgente.subpartidas(@agente.id,  @fecha_de_inicio, @fecha_de_fin, params[:moneda], params[:tasa_cambio] )  
     @titulo_gasto_subpartida = ["Detalle del gasto de agente por subpartidas"] + titulo_datos
   

    @tablas =[tabla_resumen]

    # Recogemos todo en una tabla común
    respond_to do |format|
      format.xls do
        @resumen = [	{:tabla => tabla_resumen},
			{:objeto => {	:tipo => "v_presupuesto_subpartida",
					:nombre => "Presupuesto por Subpartidas",
					:titulo => @titulo_presupuesto_subpartida,
					:objetos => @v_presupuesto_subpartida } },
			{:objeto => {	:tipo => "v_gasto_subpartida",
					:nombre => "Gasto por Subpartidas",
					:titulo => @titulo_gasto_subpartida,
					:objetos => @v_gasto_subpartida } }  ]
        render 'comunes_xls/resumen', :layout => false
      end if params[:fichero] == "1"
      format.html if params[:fichero] == "0"
    end
  end

  def presupuesto
    @listado_moneda = (@agente.moneda.collect{|e| [e.nombre, e.id]}) + [["Todas las monedas (T.Cambio aplicada)", "todas"]]
    @listado_etapas = @agente.etapa.collect{|e| [e.nombre, e.id]}
    estado_presupuesto if params[:moneda] 
  end

  def estado_presupuesto
     condiciones = {"ocultar_agente" => false} unless params[:mostrar_partidas_ocultas] == "1"
     partidas = Partida.find(:all, :conditions => condiciones, :order => "codigo")
     # Recogemos los datos de la tabla primera resumen del presupuesto de agente frente al presupuesto de proyecto
     filas =  partidas.collect {|i| {"id" => i.id, "nombre" => i.codigo_nombre, "url" => {:action => "subpartida", :etapa =>  params[:etapa], :moneda => params[:moneda], :tasa_cambio => ((params[:tasa_cambio] == "1" or params[:moneda] == "todas") ? "1": "0" ), :partida => i.id}, "update" => "subpartida"}} if params[:detallado] == "0"
     filas =  partidas.collect {|i| {"id" => i.id, "nombre" => i.codigo_nombre} }   if params[:detallado] == "1"       
     columnas = [{"id" => "2", "nombre" => "Pto. Proyectos"}, {"id" => "1", "nombre" => "Pto. Agente"}]
     etapa = Etapa.find params[:etapa]
     presupuestos_proyectos = VPresupuestoDetallado.partida_de_implementador_sin_proyecto(params[:moneda], @agente.id, etapa.fecha_inicio, etapa.fecha_fin , params[:tasa_cambio], nil, true).collect {|p| {"columna_id" => "2", "importe" => p.importe , "fila_id" => p.fila_id }}
     presupuestos_delegacion = VPresupuestoAgente.partida_implementador(@agente.id, etapa.id, params[:moneda], params[:tasa_cambio]).collect {|p| {"columna_id" => "1", "importe" => p.importe , "fila_id" => p.fila_id }}
     datos = presupuestos_proyectos + presupuestos_delegacion
     titulo =[ "Moneda: #{ params[:moneda] == 'todas' ?  'Todas las monedas' : Moneda.find(params[:moneda]).abreviatura} " ,
               "Tasa cambio: #{(params[:tasa_cambio] == '1' or params[:moneda] == 'todas') ? 'Aplicada': 'No aplicada'}" ,
               "Etapa: #{ etapa.nombre + ':' + etapa.fecha_inicio.to_s + '/' + etapa.fecha_fin.to_s } " ]
     titulo_resumen = ["Resumen del presupuesto de proyectos (ingresos) y de agente (gastos)"] + titulo
     tabla = { :titulo => titulo_resumen, :filas => filas, :columnas => columnas, :datos => datos, :otros => { :fila_suma => true, :columna_resta => true } }

     # Comprobamos que no hay partidas ocultas con datos de agente
     presupuestos_proyectos.each do |p|
       flash.now[:error] = "Existen partidas ocultas que contienen presupuestos de proyecto. Seleccione mostrar partidas ocultas para verlas"  if (filas.select{|f| f["id"].to_s == p["fila_id"].to_s}).empty?
     end
     presupuestos_delegacion.each do |p|
       flash.now[:error] = "Existen partidas ocultas que contienen presupuestos de agente. Seleccione mostrar partidas ocultas para verlas"  if (filas.select{|f| f["id"].to_s == p["fila_id"].to_s}).empty?
     end

     #Recogemos la tabla del detalle del presupuesto del agente
     columnas_presupuesto_agente = [{"id" => "1", "nombre" => "Subpartida"}, {"id" => "2", "nombre" => "Importe"}]
     filas_presupuesto_agente = Presupuesto.find(:all, :select => "subpartida.nombre, partida.id", :include => [:subpartida], :conditions => {"agente_id" => @agente.id, "proyecto_id" => nil, "etapa_id" => params[:etapa]} , :group => "subpartida.nombre" ).collect{ |p| ({"nombre" => p.partida.codigo_nombre, "id" =>  (p.subpartida ? p.subpartida.id : nil) }) if p.partida }
     subpartidas = (Presupuesto.find(:all, :select => "subpartida.nombre", :include => [:subpartida], :conditions => {"agente_id" => @agente.id, "proyecto_id" => nil, "etapa_id" => params[:etapa] } , :group => "subpartida.nombre" ).collect{ |p| ({"importe" => p.subpartida.nombre, "columna_id" => "1" , "fila_id" =>  p.subpartida.id.to_s } if p.subpartida)}).compact
     presupuesto_agente_detallado = VPresupuestoAgente.subpartidas(@agente.id, params[:etapa], params[:moneda], params[:tasa_cambio] ).collect{|p| {"columna_id" => "2", "importe" => p.importe , "fila_id" => p.fila_id }}
     datos_presupuesto_agente = presupuesto_agente_detallado + subpartidas
     titulo_presupuesto_agente = ["Detalle del presupuesto del agente (gastos) por subpartidas"] + titulo
     tabla_presupuesto_agente = { :titulo => titulo_presupuesto_agente, :filas => filas_presupuesto_agente, :columnas => columnas_presupuesto_agente, :datos => datos_presupuesto_agente, :otros => { :fila_suma => true, :clases => ["","1","1_td"] } }

     #Recogemos la tabla del detalle del presupuesto de proyectos
     columnas_presupuesto_proyecto = [{"id" => "1", "nombre" => "Proyecto"}, {"id" => "2", "nombre" => "Importe"}, {"id" => "3", "nombre" => "Concepto"}]
     presupuesto_proyecto = VPresupuestoDetallado.implementador_sin_proyecto(params[:moneda], @agente.id, etapa.fecha_inicio, etapa.fecha_fin , params[:tasa_cambio], nil, true)
     filas_presupuesto_proyecto = presupuesto_proyecto.collect{ |p| ({"nombre" =>  (p.partida_codigo + " " + p.partida_nombre), "id" =>  p.presupuesto_id })}
     proyecto = presupuesto_proyecto.collect{ |p| {"importe" => p.proyecto_nombre, "columna_id" => "1" , "fila_id" =>  p.presupuesto_id.to_s } }
     presupuesto_proyecto_detallado  = presupuesto_proyecto.collect {|p| {"columna_id" => "2", "importe" => p.importe , "fila_id" => p.presupuesto_id.to_s }}
     conceptos = presupuesto_proyecto.collect{|p| {"columna_id" => "3", "importe" => p.presupuesto_concepto , "fila_id" => p.presupuesto_id.to_s }}
     datos_presupuesto_proyecto = presupuesto_proyecto_detallado + proyecto  + conceptos
     titulo_presupuesto_proyecto = ["Detalle del presupuesto por proyectos (ingresos)"] + titulo
     tabla_presupuesto_proyecto = { :titulo => titulo_presupuesto_proyecto, :filas => filas_presupuesto_proyecto, :columnas => columnas_presupuesto_proyecto, :datos => datos_presupuesto_proyecto, :otros => { :fila_suma => true, :clases => ["","1","1_td"] } }

     # Recogemos todo en una tabla común
     @tablas = [tabla, tabla_presupuesto_agente, tabla_presupuesto_proyecto]
     respond_to do |format|
        format.xls do
          @nombre = "Resumen Presupuesto" + @agente.nombre
          render 'comunes_xls/resumen_tabla', :layout => false
        end if params[:fichero] == "1"
        format.html if params[:fichero] == "0" 
      end
  end

  def subpartida
   @columnas_agente = [{"id" => "1", "nombre" => "Importe"}]
   @filas_agente = Presupuesto.find(:all, :select => "subpartida.nombre", :include => [:subpartida], :conditions => {"agente_id" => @agente.id, "partida_id" => params[:partida], :proyecto_id => nil} , :group => "subpartida.nombre" ).collect{ |p| ({"nombre" => p.subpartida.nombre, "id" =>  p.subpartida.id }) if p.subpartida }
   @etapa = Etapa.find(params[:etapa])
   @datos_agente = VPresupuestoAgente.subpartidas_partida(@agente.id, params[:etapa], params[:moneda], params[:tasa_cambio], params[:partida]).collect{|p| {"columna_id" => "1", "importe" => p.importe , "fila_id" => p.fila_id }}

   @columnas_proyecto = [{"id" => "1", "nombre" => "Importe"} ]
   presupuesto_proyecto = VPresupuestoDetallado.implementador_sin_proyecto(params[:moneda], @agente.id, @etapa.fecha_inicio, @etapa.fecha_fin , params[:tasa_cambio], params[:partida])
   @filas_proyecto = presupuesto_proyecto.collect {|p| {"nombre" => p.proyecto_nombre , "id" => p.presupuesto_id}}
   @datos_proyecto  = presupuesto_proyecto.collect {|p| {"columna_id" => "1" , "fila_id" => p.presupuesto_id.to_s, "importe" => p.importe}}
  
   render :partial => "subpartida" 
  end



        # en agentes: selecciona la caja para sacar el informe de arqueo
  def arqueo_caja
    @listado_libros = eval( "@" + singularizar_seccion ).libro.select{|l| session[:usuario_identificado].libro.include? l}.collect {|a|[a.nombre, a.id]}
    @listado_etapa = @agente.etapa.collect{|e| [e.nombre, e.id]}

    # Si hemos especificado un libro para ver el informe
    if params[:selector] and params[:selector][:libro] != ''
      @libro = Libro.find(params[:selector][:libro])
      @moneda = @libro.moneda
      condiciones = { }
      if params[:selector][:etapa] != ''
        @etapa = Etapa.find(params[:selector][:etapa])
        condiciones[:fecha] = @etapa.fecha_inicio..@etapa.fecha_fin
      end
      # Obtiene todas las transferencias de la cuenta en el periodo
      transferencias = @libro.transferencia.find :all, :conditions => condiciones
      # Obtiene todos los gastos en el periodo
      condiciones[:agente_id] = @agente.id
      condiciones.delete(:fecha)
      condiciones["pago.fecha"] = @etapa.fecha_inicio..@etapa.fecha_fin if @etapa
      gastos = Gasto.all(:include => ["pago"], :conditions => condiciones)
      # Genera el array de gastos y transferencias para presentar
      lineas = Array.new
      filas_desglosado = Array.new
      total = Hash.new
      entrante=0
      saliente=0
      # Recorre y toma todos los datos de las transferencias
      transferencias.each do |transferencia|
        total[transferencia.tipo.capitalize] = Hash.new if total[transferencia.tipo.capitalize].nil?
        total[transferencia.tipo.capitalize][transferencia.entrante_saliente.capitalize] ||= 0
        total[transferencia.tipo.capitalize][transferencia.entrante_saliente.capitalize] += transferencia.importe
        entrante += transferencia.importe if transferencia.entrante_saliente=="entrante"
        saliente += transferencia.importe if transferencia.entrante_saliente!="entrante"
        filas_desglosado.push( :contenido => [ transferencia.fecha, transferencia.tipo.capitalize, transferencia.observaciones ? transferencia.observaciones : '',
                                                transferencia.entrante_saliente=="entrante" ? transferencia.importe : '',
                                                transferencia.entrante_saliente!="entrante" ? transferencia.importe : '' ])
      end
      # Recorre y toma todos los datos de los pagos
      gastos.each do |gasto|
        gasto.pago.each do |pago|
          if pago.libro_id == @libro.id
            total["Pago"] = Hash.new if total["Pago"].nil?
            total["Pago"]["Saliente"] = 0 if total["Pago"]["Saliente"].nil?
            total["Pago"]["Saliente"] += pago.importe
            saliente += pago.importe
            filas_desglosado.push( :contenido => [     pago.fecha, _("Pago"), (pago.observaciones||gasto.concepto),
                                                        '', pago.importe ] )
          end
        end
      end
      # Ordena por fecha
      filas_desglosado.sort! { |x,y| x[:contenido][0] <=> y[:contenido][0] }

      # Prepara la salida de datos
      lineas.push( :cabecera => [ [_("Fecha"),"1_2"], [_("Tipo"),"1_2"], [_("Concepto"),"3_2"], [_("Entradas"),"1_2_td"], [_("Salidas"),"1_2_td"] ])
      lineas += filas_desglosado
      lineas.push( :contenido => [ '','','','','' ] )
      lineas.push( :cabecera => [ [_("Totales"),"1_2"], ["","1_2"], ["","3_2"], [entrante,"1_2_td"], [saliente,"1_2_td"], [entrante - saliente,"1_2_td"] ])
      total.each do |k,v|
        lineas.push( :contenido => [ '',k,'',v["Entrante"]||'',v["Saliente"]||'','' ] )
      end

      @resumen = Array.new
      nombre = "arqueo_caja"
      titulo = "Arqueo Cuenta/Caja " + @libro.nombre + " (" + @libro.moneda.nombre + ") / Etapa: " +
      			(@etapa ? @etapa.nombre + " (desde " + @etapa.fecha_inicio.strftime('%d/%m/%Y') + " hasta " + @etapa.fecha_fin.strftime('%d/%m/%Y') + ")" : "Todas")
      @resumen.push(:listado => {:nombre => nombre, :titulo => titulo, :lineas => lineas})

      respond_to do |format|
        format.html do
          render 'comunes/arqueo_caja'
        end if params[:selector][:fichero] == "0"
        format.xls do
          render 'comunes_xls/resumen', :layout => false
        end if params[:selector][:fichero] == "1"
      end
    end

    render :template => "comunes/arqueo_caja" if params[:selector].nil?
  end

end
