#--
#
#################################################################################
# Copyright 2011 CENATIC y IEPALA
#
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas por la Comisión Europea– 
# versiones posteriores de la EUPL (la «Licencia»);
# Solo podrá usarse esta obra si se respeta la Licencia.
# Puede obtenerse una copia de la Licencia en:
#
# http://www.osor.eu/eupl/european-union-public-licence-eupl-v.1.1
#
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye «TAL CUAL»,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones que establece la Licencia.
#################################################################################
#
#++
# Controlador encargado de la gestion de proyecto. Este controlador es utilizado desde las secciones:
# * Sección inicio: visualizar las tareas asignadas al usuario identificado
# * Sección proyectos: visualizar y gestionar las tareas del proyecto

class TareaController  < ApplicationController
  # en proyectos y delegaciones: se redirecciona por defecto a ordenado
  def index
    filtrado_ordenado_iniciales
    redirect_to :action =>  "listado"
  end

  def filtrado_ordenado_iniciales
    session[:tarea_orden] = "titulo"
    session[:tarea_filtro] = "Tareas activas"
    session[:tarea_asc_desc] = "ASC"
    #redirect_to :action => "listado"
  end

  # en inicio y proyectos: establece los parametros de ordenación
  def ordenado
    session[:tarea_asc_desc] = params[:asc_desc] ? params[:asc_desc] : "ASC"   
    session[:tarea_orden] = params[:orden] ? params[:orden] : "titulo" 
    redirect_to :action => "listado"
  end

  def filtrado
    session[:tarea_filtro] = params[:filtro][:activo]
    redirect_to :action => "listado"
  end
  
  def condiciones_y_filtro
    @condiciones = Hash.new
    case session[:tarea_filtro]
      #when "Cualquier estado" then @condiciones["estado_tarea.activo"] = true 
      when "Tareas activas" then @condiciones["estado_tarea.activo"] = true 
      when "Tareas no activas" then @condiciones["estado_tarea.activo"] = false
    end 
    case params[:seccion]
      when "inicio" then @condiciones[:usuario_asignado_id] = session[:usuario_identificado].id
      when "proyectos" then @condiciones[:proyecto_id] = @proyecto.id
      when "agentes" then @condiciones[:agente_id] = @agente.id
    end
    @filtros = [ "Tareas activas", "Tareas no activas", "Cualquier estado"]  #] + EstadoTarea.find(:all).collect{ |e| [e.nombre, e.id] }.sort{ |a, b| b[0] <=> a[0]}
  end  
    
  def listado
    condiciones_y_filtro
    @tareas =  Tarea.paginate( :all, :conditions => @condiciones, :include => [:tipo_tarea, :estado_tarea], :order => ( session[:tarea_orden] + " " + session[:tarea_asc_desc] ), :page => params[:page], :per_page => (session[:por_pagina] or 20) )
    if params[:seccion] == "inicio"
      render :action => "inicio"
    else
      render :action => "listado"
    end
  end

  # en inicio: selecciona el proyecto de la tarea y lo carga en la sessión
  def ir_tareas
    tarea = Tarea.find_by_id(params[:id])
    filtrado_ordenado_iniciales
    if tarea.proyecto_id
      puts "-------> Vamos a proyectos"
      redirect_to :seccion => 'proyectos', :proyecto_id => tarea.proyecto_id, :menu => "resumen",  :controller => 'tarea', :action => "listado"
    else
      puts "-------> Vamos a agentes"
      redirect_to :seccion => 'agentes', :agente_id => tarea.agente_id, :menu => "tarea", :controller => 'tarea', :action => 'listado'
      #redirect_to :seccion => 'financiaciones', :financiacion_id => tarea.financiacion_id, :controller => 'tarea', :action => "listado"
    end
  end

  # en proyectos y delegaciones: prepara el formulario de edición o creación
  def editar_nuevo
    @tarea = Tarea.find_by_id(params[:id]) || nil
    if params[:seccion] == "proyectos"
      @usuarios = Proyecto.find_by_id(@proyecto.id).usuario.collect{ |a| [a.nombre, a.id] }
      @tipo_tarea = TipoTarea.find_all_by_tipo_proyecto( true ).collect{ |a| [a.nombre, a.id] }
    else
      @usuarios = Agente.find_by_id(@agente.id).usuario.collect{ |a| [a.nombre, a.id] }
      @tipo_tarea = TipoTarea.find_all_by_tipo_agente( true ).collect{ |a| [a.nombre, a.id] }
    end
    @estado_tarea = EstadoTarea.find(:all).collect{ |a| [a.nombre, a.id] }
    render :partial => "formulario"
  end

  # en proyectos: modifica o crea
  def modificar_crear
    @msg = ""
    proyecto_financiacion = @proyecto if params[:seccion] == "proyectos"
    proyecto_financiacion = @agente if params[:seccion] == "agentes"
    @tarea = params[:id] ?  Tarea.find(params[:id]) : Tarea.new( :fecha_inicio => Time.now() )
    #@tarea.fecha_fin = Time.now if params[:tarea][:estado] == "cerrada"
    @tarea.update_attributes params[:tarea]
    proyecto_financiacion.tarea << @tarea
    session[:usuario_identificado].tarea << @tarea
#    @usuarios = Usuario.find( :all, :conditions => { :id => @tarea.usuario_id } ) + Usuario.find( :all, :conditions => { :id => @tarea.usuario_asignado_id } ) + ( proyecto_financiacion.usuario - ( Usuario.find( :all, :conditions => { :id => @tarea.usuario_id } ) + Usuario.find( :all, :conditions => { :id => @tarea.usuario_asignado_id } ) ) )
    usuarios = Usuario.find( :all, :conditions => { :id => @tarea.usuario_id } ) + Usuario.find( :all, :conditions => { :id => @tarea.usuario_asignado_id } )
    for usuario in usuarios
      begin
        Correo.deliver_cambio_tarea(usuario, params[:seccion], @proyecto||@agente, @tarea)
      rescue
        @msg << "no se ha podido mandar el mail a " + usuario.correoe + "<br>"
      end
    end if @tarea.errors.empty?
    flash[:error] = @tarea
    redirect_to :action => "listado"
  end
  
  # en proyectos: elimina
  def eliminar
    @tarea = Tarea.find(params[:id])
    @tarea.destroy
    flash[:error_eliminar] = @tarea
    redirect_to :action => 'listado'
  end

end
#done
