#--
#
#################################################################################
# Copyright 2011 CENATIC y IEPALA
#
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas por la Comisión Europea– 
# versiones posteriores de la EUPL (la «Licencia»);
# Solo podrá usarse esta obra si se respeta la Licencia.
# Puede obtenerse una copia de la Licencia en:
#
# http://www.osor.eu/eupl/european-union-public-licence-eupl-v.1.1
#
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye «TAL CUAL»,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones que establece la Licencia.
#################################################################################
#
#++
# Controlador encargado de la gestion de Transferencias. Este controlador es utilizado desde las secciones:
# * Sección agentes: se utiliza para gestionar las transferencias de un determinado agente
# * Sección proyectos: se utiliza para gestionar las transferencias de un proyecto

class TransferenciaController < ApplicationController
  

  before_filter :verificar_estado_proyecto, :only => [ :index, :modificar_crear, :eliminar, :guardar_vinculacion]

  def verificar_estado_proyecto
    unless @proyecto.estado_actual and @proyecto.estado_actual.definicion_estado.aprobado
      flash[:error] = _("El proyecto se encuentra 'sin estado'. En este estado no se pueden modificar los datos del proyecto.")    if @proyecto.estado_actual.nil?
      flash[:error] = _("El proyecto se encuentra en estado de ") + @proyecto.estado_actual.definicion_estado.nombre + _(". En este estado no se pueden modificar los movimientos. No ha sido definido como 'aprobado' por su administrador.") unless @proyecto.estado_actual.nil?
      redirect_to :action => "listado" unless params[:action] == "index"
      redirect_to :action => :filtrado_ordenado_iniciales if params[:action] == "index"
    end if params[:seccion] == "proyectos"
  end
  
  # en proyectos y en agente: se redirecciona por defecto a filtrado_ordenado_iniciales creando el mensaje en funcion de la sección.
  def index
    objeto = singularizar_seccion
    redirect_to :action => :filtrado_ordenado_iniciales
  end

  # en proyectos y en agente: inicializa los defectos para ordenado y filtro y redirecciona a listado
  def filtrado_ordenado_iniciales
    if eval( "@" + singularizar_seccion ).etapa.empty? 
      flash[:error] = _("Tiene que definir por lo menos una etapa para acceder a la gestión de transferencias.")
      redirect_to :controller => :datos_proyecto, :action => :etapas
    else
      session[:transferencia_asc_desc] ||= "ASC" 
      session[:transferencia_cadena_orden] ||= "fecha"
      session[:transferencia_filtro_etapa] ||= "todas"
      session[:transferencia_filtro_libro] ||= "todas"
      session[:transferencia_filtro_tipo] ||= "todos"
      redirect_to :action => :listado
    end
  end

  # en proyectos y agentes: establece los parametros de ordenación
  def ordenado
    session[:transferencia_asc_desc] = params[:asc_desc] ? params[:asc_desc] : "ASC" 
    session[:transferencia_cadena_orden] = params[:orden] ? params[:orden] : "fecha"
    redirect_to :action => "listado"
  end

  # en proyectos y agentes: establece los parametros de filtro
  def filtrado
    session[:transferencia_filtro_etapa] = params[:filtro][:etapa]
    session[:transferencia_filtro_tipo] = params[:filtro][:tipo]
    session[:transferencia_filtro_libro] = params[:filtro][:libro]
    redirect_to :action => :listado
  end

  def elementos_filtrado
    @libro = Libro.find( session[:transferencia_filtro_libro] ) unless session[:transferencia_filtro_libro] == "todas"
    @etapa = Etapa.find( session[:transferencia_filtro_etapa] ) unless session[:transferencia_filtro_etapa] == "todas"
    @tipo = session[:transferencia_filtro_tipo].to_s unless session[:transferencia_filtro_tipo] == "todos"
    @entrante_saliente = session[:transferencia_filtro_entrante_saliente].to_s unless session[:transferencia_filtro_entrante_saliente] == "todas"

    filtro_etapa = [["Todas","todas"]] +   eval( "@" + singularizar_seccion ).etapa.sort{ |a, b| a.fecha_inicio <=> b.fecha_inicio }.collect{ |e| [e.nombre, e.id] }
    filtro_libro = [["Todas","todas"]] +   eval( "@" + singularizar_seccion ).libro.select{|l| session[:usuario_identificado].libro.include? l}.collect {|a|[a.nombre, a.id]}

    @opciones_filtrado = [{:rotulo =>  _("Selecciona etapa: "), :nombre => "etapa", :opciones => filtro_etapa},
                          {:rotulo =>  _("Selecciona cuenta: "), :nombre => "libro", :opciones => filtro_libro},
                          {:rotulo =>  _("Selecciona tipo: "), :nombre => "tipo", :opciones => filtro_tipo } ]

    @estado_filtrado = [(@etapa ? (@etapa.nombre + " (" + @etapa.fecha_inicio.to_s + "/" + @etapa.fecha_fin.to_s + ")") : "Cualquier etapa"),
                       (@libro ? @libro.nombre : "Cualquier cuenta/caja"),
                       (@tipo || "Cualquier tipo"), ]
    @resumen = {:url => {:action => :arqueo_caja, :controller => :resumen_proyecto, :sin_layout => true}, :mensaje => "Ver resumen de arqueo de caja"}
  end

  # en proyectos y en agente: lista
  def listado
    elementos_filtrado

    condiciones = { "libro.agente_id" => @agente.id } if params[:seccion] == "agentes"
    condiciones = { "proyecto_id" => @proyecto.id } if params[:seccion] == "proyectos"

    condiciones[:tipo] = @tipo if @tipo 
    condiciones[:libro_id] = @libro.id if @libro 
    condiciones[:fecha] = @etapa.fecha_inicio..@etapa.fecha_fin if @etapa
    #condiciones[:entrante_saliente] = "entrante" 

    @transferencias = Movimiento.paginate( :all, :conditions => condiciones,
			:include => :libro,
			:order => (session[:transferencia_cadena_orden] + " " + session[:transferencia_asc_desc] ),
			:page => (params[:format]=='xls' ? nil : params[:page]), :per_page => (params[:format_xls_count] || session[:transferencia_numero]) )

    @formato_xls = @transferencias.total_entries
    respond_to do |format|
      format.html
      format.xls do
        @tipo = "movimiento"
        @objetos = @transferencias
        render 'comunes_xls/listado', :layout => false
      end
    end

  end

	# en proyectos y agente, listar los movimientos asociados al indicado
  def listado_asociados
    transferencia = Transferencia.find_by_id params[:transferencia_id]
    @transferencia = transferencia.entrante_saliente == "saliente" ? transferencia : (transferencia.transferencia_vinculada || transferencia)
    render :update do |page|
      page.replace_html params[:update] + "_sub", :partial => "listado_asociados", :locals => { :update => params[:update] }
    end
  end

	# en proyectos y en agente: prepara el formulario de edición o creación
  def editar_nuevo
    @transferencia = params[:id] ?  Transferencia.find(params[:id]) : Transferencia.new(:tipo => params[:tipo], :transferencia_id => params[:transferencia_id], :libro_id => params[:libro_id], :libro_receptor_emisor_id => params[:libro_receptor_emisor_id], :moneda_id => params[:moneda_id], :entrante_saliente => params[:entrante_saliente])
    @etapa = Etapa.find( session[:transferencia_filtro_etapa] ) unless session[:transferencia_filtro_etapa] == "todas"
    @proyectos = @agente.proyecto_implementador.collect{ |p| [p.nombre, p.id] } if params[:seccion] == "agentes"
    @proyecto_asociado = @proyecto ? @proyecto : @transferencia.proyecto
    # Si proviene del formulario general y se ha elegido un tipo, pinta el formulario
    if params[:nuevo_tipo]
      render :update do |page|
        page.replace "MB_caption", :inline => "<div id='MB_caption'>" + ( @proyecto ? "Proyecto: " + @proyecto.nombre : "Agente: " + @agente.nombre) + "<br>Nuevo movimiento: " + params[:nuevo_tipo].capitalize + "</div>"
        page.replace params[:form_update], :partial => "formulario_" + params[:nuevo_tipo]
        page.call("Modalbox.resizeToContent");
      end 
    # Si es uno nuevo, o ya existia el movimiento pinta el formulario correspondiente
    else
      tipos = filtro_tipo
      tipos.shift
      #render :partial => "formulario" + ( @transferencia.id.nil? ? "" : "_" + @transferencia.tipo ), :locals => {:tipos => tipos}
      render :partial => "formulario" + ( @transferencia.tipo ? "_" + @transferencia.tipo : ""), :locals => {:tipos => tipos}
    end
  end

  # en proyectos y en agente: modifica o crea
  def modificar_crear
    @etapa = Etapa.find( session[:transferencia_filtro_etapa] ) unless  session[:transferencia_filtro_etapa] == "todas"
    @transferencia = params[:id] ?  Transferencia.find_by_id(params[:id]) : Transferencia.new 
    # Le asignamos el proyecto o el agente
    params[:transferencia][(singularizar_seccion + "_id").to_sym] = eval( "@" + singularizar_seccion ).id
    # Si es un movimiento saliente, no tiene tasa de cambio
    params[:transferencia][:tasa_cambio] = nil if params[:transferencia][:entrante_saliente] == "saliente"
    # Si las monedas son iguales, la tasa de cambio es 1
    if params[:transferencia][:libro_id] && params[:transferencia][:libro_receptor_emisor_id] && params[:transferencia][:entrante_saliente] == "entrante"
      params[:transferencia][:tasa_cambio] = 1 if Libro.find_by_id(params[:transferencia][:libro_id]).moneda_id == Libro.find_by_id(params[:transferencia][:libro_receptor_emisor_id]).moneda_id
    end

    # Le metemos los datos del formulario, pero sin guardar 
    #@transferencia.attributes = params[:transferencia]
    # Actualizamos los datos del movimiento 
    #@transferencia.update_attributes params[:transferencia] if (session[:transferencia_filtro_etapa] == "todas" || @transferencia.comprobar_fecha_etapa @etapa)
    @transferencia.update_attributes params[:transferencia]

    # En el caso de que sea un movimiento entrante y la cuenta destino sea tambien del usuario, generamos la salida 
    if params[:transferencia][:entrante_saliente] == "entrante" && params[:selector][:entrada] == "1" &&
                params[:transferencia][:libro_receptor_emisor_id] && session[:usuario_identificado].libro.find_by_id(params[:transferencia][:libro_receptor_emisor_id])
      # Esto hay que revisarlo!!!!
      @movimiento_vuelta = Transferencia.find_or_create_by_id(@transferencia.transferencia_id)

      @movimiento_vuelta.attributes = params[:transferencia]
      @movimiento_vuelta.entrante_saliente = "saliente"
      @movimiento_vuelta.tasa_cambio = nil
      @movimiento_vuelta.libro_id = @transferencia.libro_receptor_emisor_id
      @movimiento_vuelta.libro_receptor_emisor_id =  @transferencia.libro_id
      @movimiento_vuelta.moneda_id = Libro.find_by_id( @transferencia.libro_receptor_emisor_id ).moneda_id
      @movimiento_vuelta.importe = @transferencia.importe * @transferencia.tasa_cambio if @transferencia.importe && @transferencia.tasa_cambio
    end if params[:transferencia][:tipo] =~ /transferencia|cambio|cheque|retirada/

    # Si no hemos tenido errores guardamos el movimiento de salida 
    if @transferencia.errors.empty? && @movimiento_vuelta #.errors.empty?
      @movimiento_vuelta.save
      # Y vinculamos los 2 movimientos
      @transferencia.update_attribute :transferencia_id, @movimiento_vuelta.id
      @movimiento_vuelta.update_attribute :transferencia_id, @transferencia.id
    end

    combinado = Movimiento.new([@transferencia]) if @transferencia.errors.empty?
    #unimos los errores de las 2 entidades para su viualización
    @movimiento_vuelta.errors.each{|e,m| @transferencia.errors.add "", m} if @movimiento_vuelta
    render :update do |page| 
      page.replace 'formulario', :inline => '<%= mensaje_error(@movimiento_vuelta) %><br>' if @transferencia.errors.empty? && @movimiento_vuelta
      page.replace 'formulario', :inline => '<%= mensaje_error(@transferencia) %><br>' unless @transferencia.errors.empty? && @movimiento_vuelta
      page.call("Modalbox.resizeToContent")
      page.replace params[:update], :partial => "transferencia", :locals => { :combinado => combinado.first, :update => params[:update] } if combinado 
      page.replace_html params[:update_sub], :partial => "listado_asociados", :locals => { :update => params[:update] } if params[:update_sub] if @transferencia.errors.empty?
      page.visual_effect :highlight, params[:update] , :duration => 6
    end if params[:update]
    flash[:error] = @transferencia
    redirect_to :action => :ordenado unless params[:update]
  end

  def eliminar_transferencia
    @transf_orig = Transferencia.find_by_id(params[:id])
    @transf_orig.destroy
    @transferencia = Transferencia.find_by_id(@transf_orig.transferencia_id)
    render :update do |page|
      page.replace_html 'MB_content', :inline => '<%= mensaje_error(@transf_orig, :eliminar => true) %><br>'
      page.call("Modalbox.resizeToContent")
      page.replace params[:update], :partial => "transferencia", :locals => { :combinado => Movimiento.new([@transferencia]).first, :update => params[:update] }
      page.replace_html params[:update] + "_sub", :partial => "listado_asociados", :locals => { :update => params[:update] }
      page.visual_effect :highlight, params[:update] , :duration => 6
    end
  end

  # en proyectos y en agentes: elimina transferencias vinculadas
  def eliminar_asociados
    transf = Transferencia.find_by_id(params[:id])
    Transferencia.find_by_id(transf.transferencia_id).destroy if transf.transferencia_id
    transf.destroy
    flash[:mensaje] = _('Se han eliminado las Transferencias.')
    redirect_to :action => 'ordenado' 
  end

  # en proyectos y en agente: muestra la moneda del libro seleccionado
  def moneda_libro
    @moneda = params[:id] != "" ? Libro.find(params[:id]).moneda : nil
    render :inline => <<-FIN
      <%= _("Moneda") %> <br> <%=  @moneda.abreviatura if @moneda %>
      <%= hidden_field('transferencia', 'moneda_id', :value => @moneda.id) if @moneda%>
    FIN
  end

  # en proyectos y en agente: reacciona al cambio de libro
  def cambio_libro
    libro=Libro.find(params[:id]) if params[:id] != ""
    @moneda = libro ? libro.moneda : nil
    tipo_libro = libro.nil? ? "todos" : (libro.tipo == "banco" ? "caja \"chica\"" : "banco")
    @direccion = tipo_libro == "banco" ? params[:direccion][1] : params[:direccion][0]
    @proyecto_asociado = @proyecto ? @proyecto : @transferencia.proyecto
    render :update do |page|
      page.replace "moneda", :inline => <<-FIN
        <div class="elemento1_3" id="moneda">
          <%= _("Moneda") %> <br> <%=  @moneda.abreviatura if @moneda %>
          <%= hidden_field('transferencia', 'moneda_id', :value => @moneda.id) if @moneda %>
        </div>
      FIN
      page.replace "transferencia_entrante_saliente", :inline => <<-FIN
        <%= hidden_field('transferencia', 'entrante_saliente', :value => @direccion ) %>
      FIN
      page.hide "tasa_cambio" if @direccion == "saliente"
      page.show "tasa_cambio" unless @direccion == "saliente"
      page.replace "selector_segundo_libro", :partial => "selector_segundo_libro", :locals => { :tipo_libro => tipo_libro, :moneda => @moneda }
    end
  end

  def gastos
    @gastos = GastoXTransferencia.find_all_by_transferencia_id( params[:transferencia_id] ).collect{ |gxt| Gasto.find(gxt.gasto_id) }
    render :update do |page|
      page.replace_html params[:update], :partial => "gastos", :locals => { :update => params[:update], :transferencia_id => params[:transferencia_id], :tipo => params[:tipo], :gastos => @gastos }
    end
  end

        # en proyectos y en financiación:  prepara el formulario de edición o creación de comentarios
  def asociar_gasto
    @gastos = @proyecto.gasto.collect{ |g| [g.importe.to_s + " -- " + g.partida.nombre + " -- " + (g.concepto || "") , g.id]}
    render :partial => "asociar_gasto", :locals => { :transferencia_id => params[:transferencia_id], :update => params[:update] }
  end

        # en proyectos y en financiación:  modifica o crea un comentario
  def guardar_asociar_gasto
    @gasto_x_transferencia = GastoXTransferencia.create( :gasto_id => params[:selector][:gasto], :transferencia_id => params[:transferencia_id] )
    @gastos = GastoXTransferencia.find_all_by_transferencia_id( params[:transferencia_id] ).collect{ |gxt| Gasto.find(gxt.gasto_id) }
    render :update do |page|
      page.replace 'formulario', :inline => '<%= mensaje_error(@gasto_x_transferencia) %><br>'
      page.call("Modalbox.resizeToContent")
      page.replace_html params[:update], :partial => "gastos", :locals => { :transferencia_id => params[:transferencia_id], :gastos => @gastos, :update => params[:update] }
      page.visual_effect :highlight, params[:update] , :duration => 6
    end
  end

        # en proyectos y en financiación:  elimina un comentario
  def eliminar_gasto
    @objeto = GastoXTransferencia.find(:first, :conditions => {:gasto_id => params[:gasto_id], :transferencia_id => params[:transferencia_id]})
    not @objeto.nil? and @objeto.destroy
    @gastos = GastoXTransferencia.find_all_by_transferencia_id( params[:transferencia_id] ).collect{ |gxt| Gasto.find(gxt.gasto_id) }
    render :update do |page|
      #page.replace "gastos_" + params[:transferencia_id], :partial => "gastos", :locals => { :transferencia_id => params[:transferencia_id], :gastos => @gastos }
      page.replace_html params[:update], :partial => "gastos", :locals => { :transferencia_id => params[:transferencia_id], :gastos => @gastos, :update => params[:update]} 
      page.visual_effect :highlight, params[:update] , :duration => 6
      page.replace_html 'MB_content', :inline => '<%= mensaje_error(@objeto, :eliminar => true) %><br>'
      page.call("Modalbox.resizeToContent")
    end
  end

  def cuenta_segunda_financiador
    @proyecto_asociado = Proyecto.find(params[:id])
    render :update do |page|
      page.replace_html "cuenta_segunda_financiador", :partial => "cuenta_segunda_financiador", :locals => { :filtro_tipo => params[:filtro_tipo] } 
      page.call("Modalbox.resizeToContent")
    end 
  end

 # --
 # VINCULACION DE MOVIMIENTOS
 # ++

  def vincular_movimientos
    @transferencia = Transferencia.find params[:id]
    condiciones = { "libro.agente_id" => @agente.id } if params[:seccion] == "agentes"
    condiciones = { "proyecto_id" => @proyecto.id } if params[:seccion] == "proyectos"
    condiciones[:transferencia_id] = nil
    condiciones[:entrante_saliente] = @transferencia.entrante_saliente == "entrante" ? "saliente" : "entrante"
    condiciones[:tipo] = @transferencia.tipo
    @transferencias = Transferencia.find(:all, :conditions => condiciones, :include => :libro, :order => " fecha asc ")
    @transferencias += [@transferencia.transferencia_vinculada] if @transferencia.transferencia_vinculada
    @transferencias = @transferencias.collect { |t| [t.fecha.to_s + "  //  " + t.importe_convertido + "  //  " + t.libro.nombre, t.id] }
    render :partial => "vincular_movimientos"
  end

  def guardar_vinculacion
    @transferencia = Transferencia.find params[:id]
    @transferencia.transferencia_vinculada.update_attribute :transferencia_id, nil if @transferencia.transferencia_vinculada
    if params[:transferencia][:desvincular] == "1"
      @transferencia.update_attribute :transferencia_id, nil
    else
      @transferencia.update_attribute :transferencia_id, params[:transferencia][:transferencia_id]
      @transferencia = Transferencia.find params[:transferencia][:transferencia_id]
      @transferencia.update_attribute :transferencia_id, params[:id]
    end
    redirect_to :action => "listado"
  end

  private
    def filtro_tipo
      [ ["Todos","todos"], ["Transferencia","transferencia"],["Cambio","cambio"],["Cheque","cheque"],["Retirada","retirada"],["Adelanto","adelanto"],
                        ["Ingreso","ingreso"],["Intereses","intereses"],["Subvencion","subvencion"] ]
    end
end
