#--
#
#################################################################################
# Copyright 2011 CENATIC y IEPALA
#
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas por la Comisión Europea– 
# versiones posteriores de la EUPL (la «Licencia»);
# Solo podrá usarse esta obra si se respeta la Licencia.
# Puede obtenerse una copia de la Licencia en:
#
# http://www.osor.eu/eupl/european-union-public-licence-eupl-v.1.1
#
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye «TAL CUAL»,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones que establece la Licencia.
#################################################################################
#
#++
# agente

class Agente < ActiveRecord::Base
  belongs_to :moneda_principal, :class_name => "Moneda", :foreign_key => "moneda_id"
  belongs_to :moneda_intermedia, :class_name => "Moneda", :foreign_key => "moneda_intermedia_id"
  belongs_to :pais

  has_many :tarea, :order => "fecha_inicio"
  has_many :agente_x_moneda
  has_many :moneda, :through => :agente_x_moneda
  has_many :libro
  has_many :etapa, :dependent => :destroy, :order => "nombre"
  has_many :usuario_x_agente, :dependent => :destroy
  has_many :usuario, :through => :usuario_x_agente
  has_many :gasto_x_agente, :dependent => :destroy
  has_many :presupuesto_x_agente, :dependent => :destroy
  has_many :presupuesto
  has_many :etapa, :dependent => :destroy
  has_many :proyecto_x_financiador, :dependent => :destroy
  has_many :proyecto_x_implementador, :dependent => :destroy
  has_many :proyecto_implementador, :through => :proyecto_x_implementador, :source => :proyecto
  has_many :proyecto_financiador, :through => :proyecto_x_financiador, :source => :proyecto

  validate :nombre_mayusculas
  validates_uniqueness_of :nombre, :message => _("Nombre") +   _(" repetido.")
  validates_presence_of :nombre, :message => _("Nombre") + _(" no puede estar vacío.")
  validates_presence_of :pais_id, :message => _("País") + _(" no puede estar vacío."), :if => :implementador

  before_destroy :verificar_borrado
  after_save :cambiar_tasas_cambio

  
  def nombre_mayusculas
    self.nombre = self.nombre.upcase
  end

  private

  def cambiar_tasas_cambio
    # Para modificaciones de agente que incluyan moneda principal, actualiza las tasas de cambio
    self.etapa.each { |e| TasaCambio.cambia_moneda(e) } if id_was && moneda_id_changed?
  end

  def self.financiador_publico_y_privado
    return  find_by_nombre("OTRAS FINANCIACIONES PÚBLICAS"), find_by_nombre("OTRAS FINANCIACIONES PRIVADAS")
  end


  def verificar_borrado
    errors.add( "presupuesto",_("hay presupuestos")) unless self.presupuesto.empty?
    errors.add( "presupuesto",_("hay presupuestos")) unless self.presupuesto_x_agente.empty?
    errors.add( "gasto", _("hay gastos")) unless self.gasto_x_agente.empty?
    errors.add( "libro", _("hay libros")) unless self.libro.empty?
    errors.add_to_base( _("Un agente tiene que estar vacio para poder ser borrado.") ) unless errors.empty?
    return false unless errors.empty?
  end
  
end
  

