#--
#
#################################################################################
# Copyright 2011 CENATIC y IEPALA
#
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas por la Comisión Europea– 
# versiones posteriores de la EUPL (la «Licencia»);
# Solo podrá usarse esta obra si se respeta la Licencia.
# Puede obtenerse una copia de la Licencia en:
#
# http://www.osor.eu/eupl/european-union-public-licence-eupl-v.1.1
#
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye «TAL CUAL»,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones que establece la Licencia.
#################################################################################
#
#++

# Proxy a la entidad Transferencia agrupando movimientos entrantes y salientes relacionados.

class Movimiento

  def initialize(transferencias)
    @transferencias = transferencias
    @movimientos = Array.new
    transferencias.each do |t|
      @movimientos.push(TransferenciasAsociadas.new(:entrante => t, :saliente => t.transferencia_vinculada)) if t.entrante_saliente == "entrante"
      @movimientos.push(TransferenciasAsociadas.new(:entrante => nil, :saliente => t)) if t.entrante_saliente != "entrante" && t.transferencia_vinculada.nil?
    end
  end

  # Metodos para el will_paginate
  def total_entries 
    @transferencias.total_entries
  end
  def total_pages 
    @transferencias.total_pages
  end
  def current_page 
    @transferencias.current_page
  end
  def previous_page
    @transferencias.previous_page
  end
  def next_page
    @transferencias.next_page
  end
  def offset
    @transferencias.offset
  end
  class << self
    def paginate(*mihash)
      Movimiento.new(Transferencia.paginate(*mihash))
    end
    def find_by_id(t_id=nil)
      Transferencia.find_by_id t_id 
    end 
  end

 private
  def method_missing(method, *args, &block)
    @movimientos.send(method, *args, &block)
  end 
end

class TransferenciasAsociadas 
  def initialize(movimiento)
    @movimiento = movimiento
  end

  def entrante
    @movimiento[:entrante]
  end
  def saliente
    @movimiento[:saliente]
  end

  def vinculadas?
    !(@movimiento[:entrante].nil? || (@movimiento[:saliente].nil? && @movimiento[:entrante].tipo =~ /transferencia|cambio|cheque|retirada|ingreso/) )
  end

  # Ponemos el id como del entrante para permitir la edicion de tasas de cambio
  def id
    @movimiento[:entrante] ? @movimiento[:entrante].id : @movimiento[:saliente].id
  end
  def tipo
    @movimiento[:entrante] ? @movimiento[:entrante].tipo : @movimiento[:saliente].tipo
  end
  def fecha
    @movimiento[:entrante] ? @movimiento[:entrante].fecha : @movimiento[:saliente].fecha
  end
  def tasa_cambio
    @movimiento[:entrante].tasa_cambio if @movimiento[:entrante] && @movimiento[:entrante].tipo =~ /transferencia|cambio/ && (@movimiento[:saliente].nil? || @movimiento[:entrante].moneda_id != @movimiento[:saliente].moneda_id)
  end
  def financiador
    @movimiento[:entrante] ? @movimiento[:entrante].agente : @movimiento[:saliente].agente
  end
  def texto_vinculacion
    entrante = @movimiento[:entrante]
    saliente = @movimiento[:saliente]
    cadena = self.tipo.capitalize + " " + (self.financiador ? self.financiador.nombre : "") + ": " 
    cadena += saliente.importe.to_s + " " + saliente.moneda.abreviatura + " (" + saliente.libro.nombre + ")" if saliente && saliente.moneda
    cadena += " -> " if entrante && saliente 
    cadena += entrante.importe.to_s + " " + entrante.moneda.abreviatura + " (" + entrante.libro.nombre + ")" if entrante && entrante.moneda
    return cadena
  end

 private
  def method_missing(method, *args, &block)
    @movimiento.send(method, *args, &block)
  end
end

