#--
#
#################################################################################
# Copyright 2011 CENATIC y IEPALA
#
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas por la Comisión Europea– 
# versiones posteriores de la EUPL (la «Licencia»);
# Solo podrá usarse esta obra si se respeta la Licencia.
# Puede obtenerse una copia de la Licencia en:
#
# http://www.osor.eu/eupl/european-union-public-licence-eupl-v.1.1
#
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye «TAL CUAL»,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones que establece la Licencia.
#################################################################################
#
#++
class Partida < ActiveRecord::Base

  has_many :gasto
  has_many :presupuesto
  # has_and_belongs_to_many :partida_financiacion, :join_table =>  "partida_x_partida_financiacion"
  has_many :partida_financiacion, :through => :partida_x_partida_financiacion, :uniq => true
  has_many :partida_x_partida_financiacion
  validate :codigo_mayusculas
  validates_presence_of :nombre, :message => _("Nombre") + _(" no puede estar vacío.")
  validates_presence_of :codigo, :message => _("Código") + _(" no puede estar vacío.")
  validates_uniqueness_of :nombre, :message => _("Nombre") + _(" repetido")
  validates_uniqueness_of :codigo, :message => _("Código") + _(" repetido")
  # TODO (validaciones) cerciorarse de que no hay ningun gasto ni presupuesto asociado para eliminar (actualizar?). 
  before_destroy :verificar_borrado

  def subpartida proyecto_id
    subpartidas = Array.new
    presupuesto.find_all_by_proyecto_id(proyecto_id).each do |pre|
      subpartidas.push( pre.subpartida )
    end
    return subpartidas.uniq
  end


  def codigo_mayusculas
    self.codigo = self.codigo.upcase
  end

  # devuelve codigo y nombre en una misma cadena.
  def codigo_nombre proyecto_id=nil
    texto = ""
    if proyecto_id
      partida = partida_financiacion.find(:first, :conditions => {"proyecto_id" => proyecto_id})
      texto += "(" + partida.codigo + ") " if partida 
    end
    texto += self.codigo + "  " + self.nombre
    return texto 
  end

  def tipo_mayusculas
    self.tipo.capitalize
  end

  # devuelve las partidas del sistema que no fueron asociadas a ninguna partida de financiación
  def self.partidas_pendientes_de_financiacion proyecto_id
    partidas = Partida.find( :all, :include => "partida_financiacion", :conditions => ["partida_financiacion.proyecto_id = ?", proyecto_id] ).collect{ |p| p.id }
    conditions = [ "id NOT IN (?)", partidas]  if ! partidas.empty?
    partidas_pendiente_financiacion = Partida.find(:all, :order => "partida.codigo", :conditions => conditions )
  end

  # comprueba si la partida esta asociada a alguna partida de financiacion
  def partida_asociada proyecto
    partida_financiacion.find :first, :conditions => {"proyecto_id" => proyecto}
  end

  def verificar_borrado
    errors.add( "presupuesto",_("hay presupuestos")) unless self.presupuesto.empty?
    errors.add( "gasto", _("hay gastos")) unless self.gasto.empty?
    errors.add( "partida_financiacion", _("hay partidas de financiación")) unless partida_financiacion.empty?
    errors.add_to_base( _("Una partida tiene que estar vacio para poder ser borrado.") ) unless errors.empty?
    return false unless errors.empty?
 end
end
