#--
#
#################################################################################
# Copyright 2011 CENATIC y IEPALA
#
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas por la Comisión Europea– 
# versiones posteriores de la EUPL (la «Licencia»);
# Solo podrá usarse esta obra si se respeta la Licencia.
# Puede obtenerse una copia de la Licencia en:
#
# http://www.osor.eu/eupl/european-union-public-licence-eupl-v.1.1
#
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye «TAL CUAL»,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones que establece la Licencia.
#################################################################################
#
#++
class PartidaFinanciacion < ActiveRecord::Base

  untranslate_all
  belongs_to :proyecto 
  has_many :partida, :through => :partida_x_partida_financiacion, :uniq => true
  has_many :partida_x_partida_financiacion, :dependent => :destroy
  validates_presence_of :nombre, :message => _("Nombre") + _(" no puede estar vacío.")
  validates_presence_of :codigo, :message => _("Código") + _(" no puede estar vacío.")
  validates_uniqueness_of :nombre, :scope => :proyecto_id, :message => _("Nombre") + _(" repetido.")
  validates_uniqueness_of :codigo, :scope => :proyecto_id, :message => _("Código") + _(" repetido.")
  before_validation_on_update :comprobar_partida_financiacion_padre

  after_create :crear_asociacion_pacs
  after_update :modificar_asociacion_pacs
  after_destroy :eliminar_asociacion_pacs

  def subpartida proyecto_id
    subpartidas = Array.new
    for part in partida
      part.presupuesto.find_all_by_proyecto_id(proyecto_id).each do |pre|
        subpartidas.push( pre.subpartida )
      end
    end
    return subpartidas.uniq
  end

  def codigo_nombre 
    return self.codigo + "  " + self.nombre 
  end

  def tipo_mayusculas
    self.tipo.capitalize
  end

  def comprobar_partida_financiacion_padre
    errors.add( "partida_financiacion_id", _("La partida superior no puede ser la misma partida"))  if partida_financiacion_id == id
    return false unless errors.empty?
  end

 def suma_presupuesto
    Presupuesto.sum "importe * tasa_cambio",  :include => ["partida_x_partida_financiacion", "tasa_cambio"], :conditions => { "partida_x_partida_financiacion.partida_financiacion_id" => id, "presupuesto.proyecto_id" => proyecto_id }
  end

 def suma_presupuesto_financiador financiador_id
    Presupuesto.sum "presupuesto_x_agente.importe * tasa_cambio",  :include => ["partida_x_partida_financiacion", "tasa_cambio", "presupuesto_x_agente"], :conditions => { "partida_x_partida_financiacion.partida_financiacion_id" => id, "presupuesto.proyecto_id" => proyecto_id, "presupuesto_x_agente.agente_id" => financiador_id }
  end

  def numero_presupuesto 
    Presupuesto.count  :include => ["partida_x_partida_financiacion", "tasa_cambio"], :conditions => { "partida_x_partida_financiacion.partida_financiacion_id" => id, "presupuesto.proyecto_id" => proyecto_id }
  end

 def porcentaje_presupuesto 
    total = Presupuesto.sum "importe * tasa_cambio", :include => ["tasa_cambio"], :conditions => {"presupuesto.proyecto_id" => proyecto_id }
    (suma_presupuesto.to_f / total.to_f) * 100
 end

 def porcentaje_presupuesto_financiador financiador_id
    total = Presupuesto.sum "presupuesto_x_agente.importe * tasa_cambio", :include => ["tasa_cambio", "presupuesto_x_agente"], :conditions => {"presupuesto.proyecto_id" => proyecto_id, "presupuesto_x_agente.agente_id" => financiador_id  }
    (suma_presupuesto_financiador(financiador_id).to_f / total.to_f) * 100
 end


 private
  def crear_asociacion_pacs
    self.proyecto.pacs.each do |p|
      mipf = self.clone
      mipf.proyecto_id = p.id
      mipf.save
    end if self.proyecto.convenio?
  end

  def modificar_asociacion_pacs
    self.proyecto.pacs.each do |p|
      mipf = p.partida_financiacion.find_by_codigo(self.codigo_was)
      mipf.attributes = self.attributes
      mipf.proyecto_id = p.id
      mipf.save
    end if self.proyecto.convenio?
  end

  def eliminar_asociacion_pacs
    self.proyecto.pacs.each { |p| PartidaFinanciacion.destroy_all(:proyecto_id => p.id, :codigo => self.codigo) } if self.proyecto.convenio?
  end

 
end
