class ResumenPresupuestoAgente < ActiveRecord::Migration
  def self.up
  

    remove_column :presupuesto, :tipo_proyecto
    remove_column :presupuesto, :tipo_delegacion

    execute "DROP VIEW IF EXISTS `v_presupuesto_detallado`"
    execute "DROP VIEW IF EXISTS `v_presupuesto`"
    execute "DROP VIEW IF EXISTS `v_presupuesto_agente`"

    execute <<-__EOI
CREATE ALGORITHM=UNDEFINED DEFINER=CURRENT_USER SQL SECURITY DEFINER VIEW `v_presupuesto_detallado` AS 


select 
A.id as financiador_id,A.nombre as financiador_nombre,
AC.id as actividad_id,AC.codigo as actividad_codigo,AC.descripcion as actividad_nombre,
P.agente_id as implementador_id, AIMP.nombre as implementador_nombre,
PY.id as proyecto_id, PY.nombre as proyecto_nombre,
PT.id as partida_id,PT.nombre as partida_nombre,
PTF.id as partida_proyecto_id,PTF.nombre as partida_proyecto_nombre,
P.id as presupuesto_id,
P.moneda_id as moneda_id,
M.abreviatura as moneda_nombre,
PY.moneda_id as moneda_base_id,
MAUX.nombre as moneda_base_nombre,
IFNULL(P.tasa_cambio,0) as tasa_cambio,
PD.fecha_inicio,
PD.fecha_fin,
PD.mes,
(PD.importe * ((PXA.importe * PXAC.importe)/P.importe))/P.importe as importe,
((PD.importe * ((PXA.importe * PXAC.importe)/P.importe))/P.importe) * P.tasa_cambio as importe_moneda_base

from 

presupuesto P
inner join presupuesto_detallado PD on P.id = PD.presupuesto_id
inner join moneda M on P.moneda_id = M.id
inner join agente AIMP on P.agente_id = AIMP.id
inner join partida PT on P.partida_id = PT.id
inner join presupuesto_x_actividad PXAC on P.id = PXAC.presupuesto_id
inner join actividad AC on PXAC.actividad_id = AC.id
inner join presupuesto_x_agente PXA on PXA.presupuesto_id = P.id
inner join agente A on PXA.agente_id = A.id
inner join proyecto PY on P.proyecto_id = PY.id
inner join moneda MAUX on MAUX.id = PY.moneda_id
left join partida_x_partida_financiacion PXPA on PT.id = PXPA.partida_id
left join partida_financiacion PTF on PTF.id = PXPA.partida_financiacion_id
__EOI

    execute <<-__EOI
CREATE ALGORITHM=UNDEFINED DEFINER=CURRENT_USER SQL SECURITY DEFINER VIEW `v_presupuesto_agente` AS 


select 
P.agente_id as implementador_id, AIMP.nombre as implementador_nombre,
E.id as etapa_id,E.nombre as etapa_nombre,
P.id as presupuesto_id,
P.moneda_id as moneda_id,
P.proyecto_id as proyecto_id,
M.abreviatura as moneda_nombre,
AIMP.moneda_id as moneda_base_id,
MAUX.nombre as moneda_base_nombre,
SP.nombre as subpartida_nombre,
P.subpartida_id as subpartida_id,
IFNULL(P.tasa_cambio,0) as tasa_cambio,
P.importe as importe,
P.importe * P.tasa_cambio as importe_moneda_base,
PT.id as partida_id

from 
presupuesto P 
inner join moneda M on P.moneda_id = M.id
inner join agente AIMP on P.agente_id = AIMP.id
inner join etapa E on E.id = P.etapa_id
inner join partida PT on P.partida_id = PT.id
left join moneda MAUX on MAUX.id = AIMP.moneda_id
left join subpartida SP on SP.id = P.subpartida_id
;
__EOI


    execute <<-__EOI
CREATE ALGORITHM=UNDEFINED DEFINER=CURRENT_USER SQL SECURITY DEFINER VIEW `v_presupuesto` AS 


select 
A.id as financiador_id,A.nombre as financiador_nombre,
AC.id as actividad_id,AC.codigo as actividad_codigo,AC.descripcion as actividad_nombre,
P.agente_id as implementador_id, AIMP.nombre as implementador_nombre,
PY.id as proyecto_id, PY.nombre as proyecto_nombre,
E.id as etapa_id,E.nombre as etapa_nombre,
PT.id as partida_id,PT.nombre as partida_nombre,
PTF.id as partida_proyecto_id,PTF.nombre as partida_proyecto_nombre,
P.id as presupuesto_id,
P.moneda_id as moneda_id,
M.abreviatura as moneda_nombre,
PY.moneda_id as moneda_base_id,
MAUX.nombre as moneda_base_nombre,
IFNULL(P.tasa_cambio,0) as tasa_cambio,
((PXA.importe * PXAC.importe)/P.importe) as importe,
((PXA.importe * PXAC.importe)/P.importe) * P.tasa_cambio as importe_moneda_base

from 
presupuesto P 
inner join presupuesto_x_actividad PXAC on PXAC.presupuesto_id = P.id
inner join actividad AC on PXAC.actividad_id = AC.id
inner join presupuesto_x_agente PXA on PXA.presupuesto_id = P.id
inner join agente A on PXA.agente_id = A.id
inner join moneda M on P.moneda_id = M.id
inner join agente AIMP on P.agente_id = AIMP.id
inner join etapa E on  E.id = P.etapa_id
inner join partida PT on P.partida_id = PT.id
left join proyecto PY on P.proyecto_id = PY.id
left join moneda MAUX on MAUX.id = PY.moneda_id
left join partida_x_partida_financiacion PXPA on PT.id = PXPA.partida_id
left join partida_financiacion PTF on PTF.id = PXPA.partida_financiacion_id
;
__EOI


  end

  def self.down
    add_column :presupuesto, :tipo_delegacion, :boolean
    add_column :presupuesto, :tipo_proyecto, :boolean
    execute "DROP VIEW IF EXISTS `v_presupuesto_detallado`"
    execute "DROP VIEW IF EXISTS `v_presupuesto`"
    execute "DROP VIEW IF EXISTS `v_presupuesto_agente`"
  end	
end
